# waves
Waves is a collection of general utilities and finite element tools for solving various PDE. The code is written in C++ interfaced in python through SWIG, and is developed at the University of Liège.

![](/dox/img.png)

## Features
Set of python/C++ modules:
  - [fwk](/fwk): general utilities (timers, ...)
  - [tbox](/tbox): general finite element utilities (mesh handler, nodes/elements handler, ...)
  - [tboxVtk](/tboxVtk): VTK extensions for tbox
  - [waves](/waves): 3D(Hex8) transient wave equation solver (FEM)
  - [fdtd](/fdtd): 2D Maxwell equations (FD)
  - [flow](/flow): 2D(Tri3) and 3D(Tetra4) steady full potential equation solver (FEM)
  - [heat](/heat): 2D(Tri3) steady nonlinear heat equation solver (multiscale FE²M)
  - [mirrors](/mirrors): 3D(Hex8 and Tetra4) linear thermomechanical equations (FEM)
  - [katropton](/katropton): 3D(Hex8 and Tetra4) linear thermomechanical equations with contact (FEM)
  - [params](/params): parametric study with Metafor
  - [tlnos](/tlnos): basic Trilinos examples
  - [sph](/sph): incompressible fluid equations (SPH)

Additional features:
  - [x] [gmsh](http://geuz.org/gmsh/) support for mesh and post-processing
  - [x] [vtk](http://www.vtk.org/)/[PyQt](https://riverbankcomputing.com/software/pyqt/intro) support for post processing and basic GUI
  - [x] [OpenBLAS](http://www.openblas.net/) or [Intel MKL](https://software.intel.com/en-us/intel-mkl) support
  - [x] [Eigen3](http://eigen.tuxfamily.org/index.php?title=Main_Page) support for linear algebra
     - [MUMPS](http://mumps.enseeiht.fr/) support
  - [x] [TBB](https://www.threadingbuildingblocks.org/) support for multithreading
  - [x] [mpi4py](https://bitbucket.org/mpi4py/mpi4py) support for parallelization
  - [x] [Trilinos](https://trilinos.github.io/) support (for some modules)

## Build

Detailed build instructions can be found in the [wiki](https://gitlab.uliege.be/am-dept/waves/wikis/home).

## References

Kim thesis

Adrien thesis

Kim Paper

Crovato A., Boman R., Guner H., Terrapon V., Dimitriadis G., Almeida H., Prado A., Breviglieri C., Cabral P., and Silva, G., [A Full Potential Static Aeroelastic Solver for Preliminary Aircraft Design](http://hdl.handle.net/2268/237955), 18th International Forum on Aeroelasticity and Structural Dynamics, IFASD 2019. 
