/*
 * Copyright 2020 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef WSOLVER_H
#define WSOLVER_H

#include "flow.h"
#include "wObject.h"
#include "wProblem.h"

#include <iostream>
#include <vector>
#include <memory>

namespace flow
{

/**
 * @brief Base solver class
 * @authors Adrien Crovato
 */
class FLOW_API Solver : public fwk::wSharedObject
{
public:
    std::shared_ptr<Problem> pbl; ///< problem definition

    int nthreads;  ///< number of threads for the execution
    double relTol; ///< relative tolerance on the residual
    double absTol; ///< absolute tolerance on the residual
    int maxIt;     ///< max number of iterations
    bool verbose;  ///< display more info

    int nIt;                  ///< number of iterations
    std::vector<double> phi;  ///< full potential
    std::vector<double> rPhi; ///< residual on potential
    std::vector<double> vPhi; ///< perturbation potential
    std::vector<tbox::Pt> U;  ///< velocity
    std::vector<double> rho;  ///< density
    std::vector<double> M;    ///< mach number
    std::vector<double> Cp;   ///< pressure coefficient
    double Cl;                ///< lift coefficient
    double Cd;                ///< drag coefficient
    double Cm;                ///< pitch moment coefficient (positive nose-up)

public:
    Solver(std::shared_ptr<Problem> _pbl);
    ~Solver();

    virtual bool run();
    void save(int n, std::shared_ptr<tbox::MshExport> mshWriter);

protected:
    void computeFlow();
    void computeLoad();
};

} // namespace flow
#endif //WSOLVER_H
