#!/usr/bin/env python
# -*- coding: utf-8 -*-
# test periodic BCs

from __future__ import print_function
import heat as h
import tbox
import tbox.gmsh as gmsh

def main():
    msh = gmsh.MeshLoader("honeycomb_jalil.geo", __file__).execute()
    pbl = h.Problem(msh)

    solver = h.Solver(pbl)
    solver.nthreads = 6

    fa=tbox.PwLf()
    fa.add(0, 80)
    fa.add(300, 80)
    fa.add(350, 20)
    fa.add(450, 10)
    fa.add(550, 5)
    fa.add(750, 1)
    fa.add(1000, 1)

    fs=tbox.PwLf()
    fs.add(0, 140)
    fs.add(300, 140)
    fs.add(350, 180)
    fs.add(450, 200)
    fs.add(550, 210)
    fs.add(750, 220)
    fs.add(1000, 220)

    fz = tbox.PwLf()
    fz.add(0,0)

    h.Medium(pbl, "Air", tbox.Fct2PwLf(fa,fa,fz), 1.)
    h.Medium(pbl, "Skeleton", tbox.Fct2PwLf(fs,fs,fz), 1.)

    TM   = 600.0
    dTxM = 0.
    dTyM = 200.
    p = h.Periodic(pbl, TM, dTxM, dTyM)

    solver.restol = 1e-6
    solver.start()

    print("mean flux=", p.qM)

    from heat.viewer import GUI
    GUI().open('honeycomb_jalil')

if __name__ == "__main__":
    main()
