#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import print_function
from __future__ import division
from builtins import str
from builtins import range
from past.utils import old_div
from katropton.eigenvalues import *
from katropton.readers import *
from katropton.readers import *
import os
import fwk
"""@package docstring
This file is used to investigate the choice of prec parameters.
"""


def read_timer_all(n_cases, sub_cases, file_MC_base_dir):
    querylines = ['Belos: Operation Op*x',
                  'Belos: Operation Prec*x',
                  'Belos: DGKS[2]: Orthogonalization',
                  'Belos: DGKS[2]: Ortho (Inner Product)',
                  'Belos: DGKS[2]: Ortho (Norm)',
                  'Belos: DGKS[2]: Ortho (Update)',
                  'Belos: PseudoBlockGmresSolMgr total solve time']
    timer = np.zeros((n_cases, len(sub_cases), len(querylines)))
    for i in range(0, n_cases):
        file_MC_base_dir_2 = file_MC_base_dir + '/' + str(i)+'/'
        for j in range(0, len(sub_cases)):
            input_MC_file_dir = file_MC_base_dir_2 + str(sub_cases[j])+'/'
            timer[i, j, :] = read_teuchos_timers(
                input_MC_file_dir+'teuchos_timers.txt', querylines)
    return timer


def read_iter_all(n_cases, sub_cases, file_MC_base_dir):
    iter = np.zeros((n_cases, len(sub_cases)))
    for i in range(0, n_cases):
        file_MC_base_dir_2 = file_MC_base_dir + '/' + str(i)+'/'
        for j in range(0, len(sub_cases)):
            input_MC_file_dir = file_MC_base_dir_2 + str(sub_cases[j])+'/'
            iter_indices, residuals = read_Belos(
                input_MC_file_dir + 'belos_out.txt', 1)
            iter[i][j] = iter_indices[-1]
    return iter


def read_timer_tensor(n_cases, sub_cases, file_MC_base_dir, sizes):
    import numpy as np
    timer = read_timer_all(n_cases, sub_cases, file_MC_base_dir)

    n_0 = sizes[0]
    n_1 = sizes[1]
    n_2 = sizes[2]
    n_3 = sizes[3]

    timer_tensor = np.zeros(
        (n_0, n_1, n_2, n_3, len(sub_cases), timer.shape[2]))

    i = 0

    for i_0 in range(0, n_0):
        for i_1 in range(0, n_1):
            for i_2 in range(0, n_2):
                for i_3 in range(0, n_3):
                    if i < timer.shape[0]:
                        timer_tensor[i_0, i_1, i_2, i_3, :, :] = timer[i, :, :]
                        i = i + 1
                    else:
                        return timer_tensor

    return timer_tensor


def read_iter_tensor(n_cases, sub_cases, file_MC_base_dir, sizes):
    import numpy as np
    iter = read_iter_all(n_cases, sub_cases, file_MC_base_dir)

    n_0 = sizes[0]
    n_1 = sizes[1]
    n_2 = sizes[2]
    n_3 = sizes[3]

    iter_tensor = np.zeros((n_0, n_1, n_2, n_3, len(sub_cases)))

    i = 0

    for i_0 in range(0, n_0):
        for i_1 in range(0, n_1):
            for i_2 in range(0, n_2):
                for i_3 in range(0, n_3):
                    if i < iter.shape[0]:
                        iter_tensor[i_0, i_1, i_2, i_3, :] = iter[i, :]
                        i = i + 1
                    else:
                        return iter_tensor

    return iter_tensor


def main():
    file_dir = os.path.dirname(__file__)
    work_dir = os.getcwd()

    test = 1
    EP = True
    if test == 1:
        file_MC_base_dir = '/Volumes/HD_SONY/CMAME/Plate/gaussian/katropton_CMAME_tests_plate_mean_prec_GS_direct/'
        #file_dir + '/../../../workspace/katropton_CMAME_tests_plate_mean_prec_GS_direct/'
        sub_cases = ['mean_case', 'ensemble_reduction_8']

        GS_sweeps_00_min = 16
        GS_sweeps_00_max = 41
        delta_GS_sweeps_00 = 2

        N_GS_sweeps_00 = int(
            np.ceil(
                old_div((GS_sweeps_00_max - GS_sweeps_00_min * 1.), delta_GS_sweeps_00)))

        # Damping has to be between 0 and 2
        GS_damping_00_min = 1.75
        GS_damping_00_max = 1.9
        N_GS_damping_00 = 8

        # Omega has to be between 0 and 1
        SS_omega_min = 0.001
        SS_omega_max = 0.5
        N_SS_omega = 12

        SS_damping_min = 1.
        SS_damping_max = 1.2
        N_SS_damping = 1

        n_cases = 1248
    elif test == 2:
        file_MC_base_dir = file_dir + \
            '/../../../workspace/katropton_CMAME_tests_plate_mean_prec_GS_direct_alphabeta_1/'
        sub_cases = ['mean_case', 'ensemble_reduction_8']

        GS_sweeps_00_min = 16
        GS_sweeps_00_max = 41
        delta_GS_sweeps_00 = 2

        N_GS_sweeps_00 = int(
            np.ceil(
                old_div((GS_sweeps_00_max - GS_sweeps_00_min * 1.), delta_GS_sweeps_00)))

        # Damping has to be between 0 and 2
        GS_damping_00_min = 0.001
        GS_damping_00_max = 1.999
        N_GS_damping_00 = 20

        # Omega has to be between 0 and 1
        SS_omega_min = 1.
        SS_omega_max = 1.2
        N_SS_omega = 1

        SS_damping_min = 1.
        SS_damping_max = 1.2
        N_SS_damping = 1

        n_cases = 260

    elif test == 3:
        file_MC_base_dir = '/Volumes/HD_SONY/CMAME/Plate/katropton_CMAME_tests_plate_mean_prec_GS_direct_lvl/'
        sub_cases = ['mean_case']

        GS_sweeps_00_min = 16
        GS_sweeps_00_max = 17
        delta_GS_sweeps_00 = 1

        N_GS_sweeps_00 = int(
            np.ceil(
                old_div((GS_sweeps_00_max - GS_sweeps_00_min * 1.), delta_GS_sweeps_00)))

        # Damping has to be between 0 and 2
        GS_damping_00_min = 0.001
        GS_damping_00_max = 0.8
        N_GS_damping_00 = 50

        # Omega has to be between 0 and 1
        SS_omega_min = 0.75
        SS_omega_max = 1.1
        N_SS_omega = 5

        SS_damping_min = 1.
        SS_damping_max = 1.
        N_SS_damping = 1

        n_cases = 181
        EP = False

    iter = read_iter_tensor(n_cases, sub_cases, file_MC_base_dir, [
                            N_GS_sweeps_00, N_GS_damping_00, N_SS_omega, N_SS_damping])
    timer = read_timer_tensor(n_cases, sub_cases, file_MC_base_dir, [
                              N_GS_sweeps_00, N_GS_damping_00, N_SS_omega, N_SS_damping])

    sweeps_00 = np.arange(
        GS_sweeps_00_min, GS_sweeps_00_max, delta_GS_sweeps_00)
    damping_00 = np.linspace(
        GS_damping_00_min, GS_damping_00_max, N_GS_damping_00)
    SS_omega = np.linspace(SS_omega_min, SS_omega_max, N_SS_omega)
    SS_damping = np.linspace(SS_damping_min, SS_damping_max, N_SS_damping)

    from mpl_toolkits.mplot3d import Axes3D
    import matplotlib.pyplot as plt
    from matplotlib import cm

    opti = np.zeros((N_GS_sweeps_00, 3), dtype=int)
    opti_2 = np.zeros((N_GS_sweeps_00, 3), dtype=int)
    opti_iter = np.zeros((N_GS_sweeps_00, 1), dtype=int)
    opti_iter_2 = np.zeros((N_GS_sweeps_00, 1), dtype=int)
    opti_iter_2_one = np.zeros((N_GS_sweeps_00, 1), dtype=int)
    R = np.zeros((N_GS_sweeps_00, 1))
    opti_time = np.zeros((N_GS_sweeps_00, 7))
    opti_time_2 = np.zeros((N_GS_sweeps_00, 7))
    opti_time_per_iter = np.zeros((N_GS_sweeps_00, 7))

    #opti_iter_8 = np.array([500,469,382,338,280,255,254,239,218,213,204,197],dtype=int)

    for i in range(0, N_GS_sweeps_00):
        tmp = iter[i, :, :, :, 0]
        tmp[tmp == 0] = 1000
        opti[i, :] = np.unravel_index(np.argmin(tmp, axis=None), tmp.shape)
        print(str(sweeps_00[i]) + ' ' + str(damping_00[opti[i, 0]]
                                            ) + ' ' + str(SS_omega[opti[i, 1]]))
        opti_iter[i] = iter[i, opti[i, 0], opti[i, 1], 0, 0]
        opti_time[i, :] = timer[i, opti[i, 0], opti[i, 1], 0, 0, :]
        if EP:
            tmp2 = iter[i, :, :, :, 1]
            opti_2[i, :] = np.unravel_index(
                np.argmin(tmp2, axis=None), tmp.shape)
            print(str(sweeps_00[i]) + ' ' + str(damping_00[opti_2[i, 0]]
                                                ) + ' ' + str(SS_omega[opti_2[i, 1]]))
            opti_iter_2[i] = iter[i, opti_2[i, 0], opti_2[i, 1], 0, 1]
            opti_iter_2_one[i] = iter[i, opti_2[i, 0], opti_2[i, 1], 0, 0]
            opti_time_2[i, :] = timer[i, opti_2[i, 0], opti_2[i, 1], 0, 1, :]
            if opti_iter_2[i] == 1000:
                R[i] = 10.
            else:
                R[i] = opti_iter_2[i]*1. / opti_iter[i]
            for j in range(0, 7):
                opti_time_per_iter[i, j] = old_div(
                    opti_time[i, j], opti_iter[i])
            #R[i] = opti_iter_8[i]*1. / opti_iter[i]

    tmp = np.zeros((N_GS_sweeps_00, 2))
    tmp[:, 0] = sweeps_00[:]
    tmp[:, 1] = np.reshape(opti_iter, (N_GS_sweeps_00,))
    np.savetxt('opti_iter.txt', tmp)
    if EP:
        tmp[:, 1] = np.reshape(opti_iter_2, (N_GS_sweeps_00,))
        np.savetxt('opti_iter_2.txt', tmp)
        tmp[:, 1] = np.reshape(opti_iter_2_one, (N_GS_sweeps_00,))
        np.savetxt('opti_iter_2_one.txt', tmp)

    plt.subplot(511)
    plt.plot(sweeps_00, opti_iter)
    plt.grid()
    plt.subplot(512)
    plt.plot(sweeps_00, opti_iter)
    if EP:
        plt.plot(sweeps_00, opti_iter_2)
        plt.plot(sweeps_00, opti_iter_2_one)
    plt.grid()
    plt.subplot(513)
    if EP:
        plt.plot(sweeps_00, R)
    plt.grid()
    plt.subplot(514)
    plt.plot(sweeps_00, damping_00[opti[:, 0]])
    if EP:
        plt.plot(sweeps_00, damping_00[opti_2[:, 0]])
    plt.grid()
    plt.subplot(515)
    plt.plot(sweeps_00, SS_omega[opti[:, 1]])
    if EP:
        plt.plot(sweeps_00, SS_omega[opti_2[:, 1]])
    plt.grid()
    print(np.argmin(opti_time[:, 6]))
    print(opti)
    plt.figure()
    for i in range(0, N_GS_sweeps_00):
        plt.subplot(2, N_GS_sweeps_00, 1+i)
        plt.plot(damping_00, iter[i, :, opti[i, 1], 0, 0])
        plt.scatter(damping_00[opti[i, 0]], opti_iter[i], c='red')
        plt.subplot(2, N_GS_sweeps_00, 1+i+N_GS_sweeps_00)
        plt.plot(SS_omega, iter[i, opti[i, 0], :, 0, 0])
        plt.scatter(SS_omega[opti[i, 1]], opti_iter[i], c='red')
    '''
    plt.figure()
    plt.plot(sweeps_00,damping_00[opti[:,0]])
    
    plt.figure()
    plt.plot(sweeps_00,SS_omega[opti[:,1]])

    plt.figure()
    for i in range(0,8):
        plt.plot(damping_00,iter[0,:,i,0,0])
    
    plt.figure()
    for i in range(0,8):
        plt.plot(SS_omega,iter[0,i,:,0,0])

    plt.figure()
    for i in range(0,6):
        plt.plot(SS_omega,iter[i,5,:,0,0])

    for i in range(0,N_GS_sweeps_00):
        fig = plt.figure()
        ax = fig.add_subplot(111, projection='3d')
        X, Y = np.meshgrid(SS_omega,damping_00)
        ax.plot_surface(X, Y, iter[i,:,:,0,0], cmap=cm.coolwarm,
                           linewidth=0, alpha=0.8)
        ax.set_zlim(0, 400)
        ax.scatter(SS_omega[opti[i,1]],damping_00[opti[i,0]],opti_iter[i], c='red')
    '''
    plt.show()


if __name__ == "__main__":
    main()
