#!/usr/bin/env python
# -*- coding: latin-1; -*-

from __future__ import print_function
from builtins import str
from builtins import range
from katropton.eigenvalues import *
from katropton.convergence import *
import os
import fwk


def main():
    try:
        import mpi4py.MPI as mpi
        comm = mpi.COMM_WORLD
        rank = comm.rank
        siz = comm.size
        name = mpi.Get_processor_name()
        status = mpi.Status()
    except:
        comm = None
        rank = 0
        siz = 1
        name = "noname"

    file_dir = os.path.dirname(__file__)
    work_dir = os.getcwd()

    N = 32

    k_max = 20

    matrix_input_file_dir = file_dir + \
        '/../../workspace/katropton_CMAME_tests_plate_MC_NO_EP/1/'
    deltas_input_file_dir = file_dir + \
        '/../../workspace/katropton_CMAME_tests_plate_deltas_omegas_direct/'

    for i in range(0, N):
        timers = fwk.Timers()

        filename_Q = matrix_input_file_dir + str(i) + '/ApCTC_mm.txt'
        filename_GT = matrix_input_file_dir + str(i) + '/B_mm.txt'
        filename_f = matrix_input_file_dir + str(i) + '/b_mm_after_bc.txt'

        timers['Read files'].start()
        Q = read_mm(filename_Q)
        Q = get_symmetrical_part(Q)
        GT = read_mm(filename_GT)
        G = GT.transpose()
        f = read_mm(filename_f, is_sparse=False)
        A = create_block(Q, G, GT, None)
        n = A.shape[0]
        x0 = np.zeros((n,))
        b = np.zeros((n,))
        b[0:len(f)] = f
        omegas = np.loadtxt(deltas_input_file_dir +
                            '/omegas_direct_direct_' + str(i) + '.txt')
        n_omegas = len(omegas)

        tmp = np.loadtxt(deltas_input_file_dir +
                         '/deltas_direct_direct_' + str(i) + '.txt')

        deltas = np.zeros((n_omegas,), dtype=np.complex128)
        deltas.real = tmp[:, 0]
        deltas.imag = tmp[:, 1]

        timers['Read files'].stop()
        print(timers)

        timers['Compute the convergence'].start()
        convergence = compute_convergence(
            k_max, n_omegas, omegas, deltas, A, b, x0, verbose=True)
        timers['Compute the basconvergenceis'].stop()
        print(timers)

        np.savetxt(work_dir+'/convergence_' + str(i) + '.txt', convergence)

        f = open('timers_' + str(i) + '.txt', 'w')
        print(timers, file=f)
        f.closed


if __name__ == "__main__":
    main()
