#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import print_function
from __future__ import division
from builtins import str
from builtins import range
from past.utils import old_div
import katropton.CMAME.model as model

import katropton as m
import tbox
import tbox.gmsh as gmsh
import shlex
import subprocess
import os
import numpy as np

from fwk.wutils import parseargs

from PyTrilinos import Teuchos

import katropton.utilities as u


def evaluate_prec(msh, comm, file_dir, muelu_xml, ensemble_size=1):

    work_dir = os.getcwd()
    args = parseargs()

    p = model.defaut_params
    p['MueLu xml use template'] = False
    p['MueLu xml from current_dir'] = True
    p['MueLu xml file name'] = '/../SIMPLE_gs_direct.xml'
    p['gamma'] = 0.

    N = ensemble_size
    num_random_variables = 20
    randomVariable = np.zeros((N, num_random_variables))
    if ensemble_size is not 1:
        nuw = 128
        p['Correlation length'] = 5.
        p['Dispersion level'] = 0.1

        p['Wavenumber cutoff'] = 8.
        p['Wavenumber discretization points'] = nuw
        p['Number Z of previously drawn samples'] = 0
        p['Number Phi of previously drawn samples'] = 0
        p['Is exp'] = False

    pm = model.plate_model(ensemble_size, randomVariable, msh, comm, file_dir,
                           work_dir, p)
    pm.run(args.k)


def evaluate_all_prec(msh, comm, file_dir, muelu_xml_in, prec_params):

    N = prec_params.shape[0]

    for i in range(0, N):
        directory = str(i)
        if not os.path.exists(directory):
            os.makedirs(directory)
        os.chdir(directory)

        current_dir = os.getcwd()

        old_strings = ['$BLOCK_00_STRIDING_INFO', '$BLOCK_11_STRIDING_INFO',
                       '$BLOCK_00_GS_sweeps', '$BLOCK_00_GS_damping',
                       '$SCHUR_omega', '$SimpleSmoother_damping']
        new_strings = [
            '3', '3',
            str(int(prec_params[i, 0])),
            str(prec_params[i, 1]),
            str(prec_params[i, 2]),
            str(prec_params[i, 3])
        ]

        print(muelu_xml_in)
        u.replace_strings(muelu_xml_in, current_dir + '/SIMPLE_gs_direct.xml',
                          old_strings, new_strings)
        print(current_dir + '/SIMPLE_gs_direct.xml')

        directory = 'mean_case'
        if not os.path.exists(directory):
            os.makedirs(directory)
        os.chdir(directory)
        evaluate_prec(msh, comm, file_dir,
                      current_dir + '/SIMPLE_gs_direct.xml')
        os.chdir('..')

        for ensemble_size in [8]:
            directory = 'ensemble_reduction_' + str(ensemble_size)
            if not os.path.exists(directory):
                os.makedirs(directory)
            os.chdir(directory)
            evaluate_prec(msh,
                          comm,
                          file_dir,
                          current_dir + '/SIMPLE_gs_direct.xml',
                          ensemble_size=ensemble_size)
            os.chdir('..')

        os.chdir('..')


def main():
    try:
        import mpi4py.MPI as mpi
        comm = mpi.COMM_WORLD
        rank = comm.rank
        siz = comm.size
        name = mpi.Get_processor_name()
        status = mpi.Status()
        print("info: MPI found")
    except:
        comm = None
        rank = 0
        siz = 1
        name = "noname"
        print("info: MPI not found => MPI disabled")

    args = parseargs()

    file_dir = os.path.dirname(__file__)
    work_dir = os.getcwd()

    # Units in the meshes are in mm
    msh = u.fuse_meshes('plate_1.msh', 'plate_1_tmp', 'plate_2.msh',
                        'plate_2_tmp', 'plate_fused', file_dir, work_dir)

    muelu_xml_in = file_dir + '/../preconditioners/SIMPLE_gs_direct.xml.in'

    GS_sweeps_00_min = 16
    GS_sweeps_00_max = 41
    delta_GS_sweeps_00 = 2

    N_GS_sweeps_00 = int(
        np.ceil(
            old_div((GS_sweeps_00_max - GS_sweeps_00_min * 1.), delta_GS_sweeps_00)))

    # Damping has to be between 0 and 2
    GS_damping_00_min = 0.001
    GS_damping_00_max = 1.999
    N_GS_damping_00 = 20

    # Omega has to be between 0 and 1
    SS_omega_min = 1.
    SS_omega_max = 1.2
    N_SS_omega = 1

    SS_damping_min = 1.
    SS_damping_max = 1.2
    N_SS_damping = 1

    sweeps_00 = np.arange(GS_sweeps_00_min, GS_sweeps_00_max,
                          delta_GS_sweeps_00)
    damping_00 = np.linspace(GS_damping_00_min, GS_damping_00_max,
                             N_GS_damping_00)
    SS_omega = np.linspace(SS_omega_min, SS_omega_max, N_SS_omega)
    SS_damping = np.linspace(SS_damping_min, SS_damping_max, N_SS_damping)

    N = N_GS_sweeps_00 * N_GS_damping_00 * N_SS_omega * N_SS_damping
    '''
    N = 12
    
    prec_params = np.array([[8, 1.75, 0.46363636363636374, 1.],
                   [10, 1.75, 0.3545454545454546, 1.],
                   [12, 1.75, 0.390909090909091, 1.],
                   [14, 1.8045454545454545, 0.42727272727272736, 1.],
                   [16, 1.8227272727272728, 0.42727272727272736, 1.],
                   [18, 1.8045454545454545, 0.3545454545454546, 1.],
                   [20, 1.8227272727272728, 0.390909090909091, 1.],
                   [22, 1.8227272727272728, 0.390909090909091, 1.],
                   [24, 1.8590909090909091, 0.390909090909091, 1.],
                   [26, 1.8590909090909091, 0.3545454545454546, 1.],
                   [28, 1.8590909090909091, 0.3545454545454546, 1.],
                   [30, 1.8772727272727272, 0.390909090909091, 1.]])

    '''
    prec_params = np.zeros((N, 4))

    i = 0

    for i_0 in range(0, N_GS_sweeps_00):
        for i_1 in range(0, N_GS_damping_00):
            for i_2 in range(0, N_SS_omega):
                for i_3 in range(0, N_SS_damping):
                    prec_params[i, :] = [
                        sweeps_00[i_0], damping_00[i_1], SS_omega[i_2],
                        SS_damping[i_3]
                    ]
                    i = i + 1

    np.savetxt('prec_params.txt', prec_params)

    evaluate_all_prec(msh, comm, file_dir, muelu_xml_in, prec_params)


if __name__ == "__main__":
    main()
