#!/usr/bin/env python
# -*- coding: latin-1; -*-

from __future__ import print_function
from __future__ import division
from builtins import str
from past.utils import old_div
import katropton.CMAME.model as model

import katropton as m
import tbox
import tbox.gmsh as gmsh
import shlex
import subprocess
import os
import numpy as np

from fwk.wutils import parseargs

from PyTrilinos import Teuchos

import katropton.utilities as u
from katropton.readers import *


def f(x):
    import sys
    try:
        import mpi4py.MPI as mpi
        comm = mpi.COMM_WORLD
        rank = comm.rank
        siz = comm.size
        name = mpi.Get_processor_name()
        status = mpi.Status()
    except:
        comm = None
        rank = 0
        siz = 1
        name = "noname"
    orig_stdout = sys.stdout
    sys.stdout = open('file', 'w')
    file_dir = os.path.dirname(__file__)
    muelu_xml_in = file_dir + '/../preconditioners/SIMPLE_gs_direct.xml.in'

    directory = 'tmp'
    if not os.path.exists(directory):
        os.makedirs(directory)
    os.chdir(directory)
    N = 8

    current_dir = os.getcwd()

    msh = u.fuse_meshes('plate_1.msh', 'plate_1_tmp', 'plate_2.msh',
                        'plate_2_tmp', 'plate_fused', file_dir, current_dir)

    old_strings = ['$BLOCK_00_STRIDING_INFO', '$BLOCK_11_STRIDING_INFO',
                   '$BLOCK_00_GS_sweeps', '$BLOCK_00_GS_damping',
                   '$SCHUR_omega', '$SimpleSmoother_damping']
    new_strings = ['3', '3', str(N), str(x[0]), str(x[1]), '0']

    u.replace_strings(muelu_xml_in, current_dir +
                      '/SIMPLE_gs_direct.xml', old_strings, new_strings)

    evaluate_prec(msh, comm, file_dir, current_dir+'/SIMPLE_gs_direct.xml')

    iter_indices, residuals = read_Belos(current_dir+'/belos_out.txt', 1)
    os.chdir('..')
    sys.stdout.close()
    sys.stdout = orig_stdout
    return iter_indices[-1]


def fp(x):
    epsi = 1e-5
    jaco = np.zeros((2, 1))
    jaco[0] = old_div((f(x+epsi*np.array([1, 0])) -
                       f(x-epsi*np.array([1, 0]))), (2*epsi))
    jaco[1] = old_div((f(x+epsi*np.array([0, 1])) -
                       f(x-epsi*np.array([0, 1]))), (2*epsi))
    return jaco


def evaluate_prec(msh, comm, file_dir, muelu_xml, ensemble_size=1):
    work_dir = os.getcwd()
    args = parseargs()

    p = model.defaut_params
    p['MueLu xml use template'] = False
    p['MueLu xml file name'] = '/SIMPLE_gs_direct.xml'

    N = ensemble_size
    num_random_variables = 20
    randomVariable = np.zeros((N, num_random_variables))
    if ensemble_size is not 1:
        nuw = 128
        p['Correlation length'] = 5.
        p['Dispersion level'] = 0.1

        p['Wavenumber cutoff'] = 8.
        p['Wavenumber discretization points'] = nuw
        p['Number Z of previously drawn samples'] = 0
        p['Number Phi of previously drawn samples'] = 0
        p['Is exp'] = False

    pm = model.plate_model(ensemble_size, randomVariable,
                           msh, comm, file_dir, work_dir, p)
    pm.run(args.k)


def main():
    from scipy import optimize
    import numpy as np

    x0 = np.array([1.7, 0.5])
    bnds = ((1., 2.), (0., 1.))
    xopt = optimize.minimize(f, x0, jac=fp, bounds=bnds, options={
                             'maxiter': 200, 'disp': 1})

    print(xopt)


if __name__ == "__main__":
    main()
