from builtins import range
import numpy as np
import matplotlib.pyplot as plt
import scipy.stats as st


def compute_kde(A):
    # Take a 2D array of m x n entries where n is the number of samples
    # and m the number of QoI

    kernel = st.gaussian_kde(A)
    return kernel


def resample_kde(kernel, n_samples):
    # Sample the QoI based on the estimated pdf

    samples = kernel.resample(size=n_samples)
    return samples


def box_prob_MC(kernel, min_bounds, max_bounds, n_samples):
    # Evaluate the probability to be in a box

    samples = resample_kde(kernel, n_samples)

    insamples = np.ones((n_samples,))
    for j in range(0, n_samples):
        for i in range(0, len(min_bounds)):
            insamples[j] = bool(insamples[j]) & (
                samples[i, j] >= min_bounds[i]) & (samples[i, j] <= max_bounds[i])

    prob = np.sum(insamples)/float(n_samples)

    return prob


def plot_kde(kernel, xbounds, ybounds, fixed_values, random_index):

    xx, yy = np.mgrid[xbounds[0]:xbounds[1]:100j, ybounds[0]:ybounds[1]:100j]

    positions_x = xx.ravel()
    positions_y = yy.ravel()

    positions = np.zeros((len(fixed_values)+2, len(positions_x)))

    fixed_i = 0

    for i in range(0, positions.shape[0]):
        if i == random_index[0]:
            positions[i, :] = positions_x
        elif i == random_index[1]:
            positions[i, :] = positions_y
        else:
            positions[i, :] = fixed_values[fixed_i] * \
                np.ones(positions[i, :].shape)
            fixed_i = fixed_i + 1

    f = np.reshape(kernel(positions).T, xx.shape)

    fig = plt.figure(figsize=(8, 8))
    ax = fig.gca()
    ax.set_xlim(xbounds[0], xbounds[1])
    ax.set_ylim(ybounds[0], ybounds[1])
    cfset = ax.contourf(xx, yy, f, cmap='coolwarm')
    ax.imshow(np.rot90(f), cmap='coolwarm', extent=[
              xbounds[0], xbounds[1], ybounds[0], ybounds[1]])
    cset = ax.contour(xx, yy, f, colors='k')
    ax.clabel(cset, inline=1, fontsize=10)
    return fig, ax
