from __future__ import print_function
from __future__ import division
from builtins import str
from builtins import range
from past.utils import old_div
import numpy as np
import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import axes3d
import matplotlib.tri as mtri

from katropton.Juelich.UQ.kde import *
import katropton.utilities as u


def convergence_qoi(qoi):
    mean_qoi = np.zeros((len(qoi), ))
    for i in range(0, len(qoi)):
        mean_qoi[i] = np.mean(qoi[0:i])
    return mean_qoi


def compute_qoi(base_dir, ensemble_size, ensemble_index, l):
    OAX, OAY = 0.0595, 0.017
    npzfile = np.load(base_dir + str(ensemble_size) + '/' +
                      str(ensemble_index) + '/mirror_data_all.npz')
    x = npzfile['x']
    y = npzfile['y']
    z = 0. * x
    dz = npzfile['dz']
    T = npzfile['T']
    tri = npzfile['tri']
    optical_coefficients, dz_residual = u.compute_RBM_Curvature_Irreg(
        x, y, z, tri, dz[:, l], "m", np.array([OAX, OAY]))
    print(optical_coefficients)
    return [
        np.max(T[:, l]),
        np.min(T[:, l]),
        np.mean(T[:, l]), 1000 * np.mean(dz[:, l]), optical_coefficients[0],
        optical_coefficients[1], optical_coefficients[2],
        optical_coefficients[3], optical_coefficients[4]
    ]  # optical_coefficients[0]]


def irregularly_plot(x,
                     y,
                     z,
                     x_min,
                     x_max,
                     y_min,
                     y_max,
                     ngridx=400,
                     ngridy=400,
                     scale=1):

    xi = np.linspace(x_min, x_max, ngridx)
    yi = np.linspace(y_min, y_max, ngridy)

    triang = mtri.Triangulation(x, y)
    interpolator = mtri.LinearTriInterpolator(triang, z)
    Xi, Yi = np.meshgrid(xi, yi)
    zi = interpolator(Xi, Yi)

    #fig = plt.figure()
    #ax = fig.gca(projection='3d')
    #ax.plot_surface(Xi, Yi, zi, cmap='jet',vmin=np.amin(z),vmax=np.amax(z))
    #ax.scatter(x,y,z)

    fig = plt.figure()
    ax = fig.gca()
    levels = np.linspace(np.amin(z * scale),
                         np.amax(z * scale),
                         16,
                         endpoint=True)

    cf = ax.contourf(Xi, Yi, zi * scale, levels, cmap="jet")
    ax.contour(xi, yi, zi * scale, levels, linewidths=0.5, colors='k')
    ax.set_aspect('equal')

    cbar = fig.colorbar(cf, ticks=levels)

    return fig, cbar


def irregularly_tol_plot(x,
                         y,
                         QoI,
                         x_min,
                         x_max,
                         y_min,
                         y_max,
                         ngridx=200,
                         ngridy=200,
                         scale=1.):

    xi = np.linspace(x_min, x_max, ngridx)
    yi = np.linspace(y_min, y_max, ngridy)

    triang = mtri.Triangulation(x, y)
    interpolator_0 = mtri.LinearTriInterpolator(triang, QoI[0, :])
    interpolator_1 = mtri.LinearTriInterpolator(triang, QoI[1, :])
    interpolator_2 = mtri.LinearTriInterpolator(triang, QoI[2, :])
    interpolator_3 = mtri.LinearTriInterpolator(triang, QoI[3, :])
    interpolator_4 = mtri.LinearTriInterpolator(triang, QoI[4, :])

    Xi, Yi = np.meshgrid(xi, yi)

    z_0 = interpolator_0(Xi, Yi)
    z_1 = interpolator_1(Xi, Yi)
    z_2 = interpolator_2(Xi, Yi)
    z_3 = interpolator_3(Xi, Yi)
    z_4 = interpolator_4(Xi, Yi)

    z_5 = np.copy(z_0)

    for i in range(0, z_0.shape[0]):
        for j in range(0, z_0.shape[1]):
            if np.isfinite(z_0[i, j]):
                z_5[i, j] = u.check_optical_tolerance(
                    [z_0[i, j], z_1[i, j], z_2[i, j], z_3[i, j], z_4[i, j]],
                    "mm",
                    scale=scale)
            else:
                z_5[i, j] = z_0[i, j]

    #fig = plt.figure()
    #ax = fig.gca(projection='3d')
    #ax.plot_surface(Xi, Yi, zi, cmap='jet',vmin=np.amin(z),vmax=np.amax(z))
    #ax.scatter(x,y,z)

    fig = plt.figure()
    ax = fig.gca()
    levels = np.linspace(0., 1., 3, endpoint=True)

    cf = ax.contourf(Xi, Yi, z_5, levels, cmap="jet")
    ax.contour(xi, yi, z_5, levels, linewidths=0.5, colors='k')
    ax.set_aspect('equal')

    cbar = fig.colorbar(cf, ticks=levels)


def clip_scater(x, y, z, x_lim, y_lim, z_lim):
    x_new = np.array([])
    y_new = np.array([])
    z_new = np.array([])
    for i in np.arange(len(x)):
        if x[i] >= x_lim[0] and x[i] <= x_lim[1] and y[i] >= y_lim[0] and y[
                i] <= y_lim[1] and z[i] >= z_lim[0] and z[i] <= z_lim[1]:
            x_new = np.append(x_new, x[i])
            y_new = np.append(y_new, y[i])
            z_new = np.append(z_new, z[i])
    return x_new, y_new, z_new


def plot_kde_2D(x, y, x_min, x_max, y_min, y_max, ngridx=200, ngridy=200):

    xi = np.linspace(x_min, x_max, ngridx)
    yi = np.linspace(y_min, y_max, ngridy)

    xx, yy = np.meshgrid(xi, yi)
    positions = np.vstack([xx.ravel(), yy.ravel()])
    values = np.vstack([x, y])
    kernel = st.gaussian_kde(values)
    f = np.reshape(kernel(positions).T, xx.shape)

    fig = plt.figure()
    ax = fig.gca()
    ax.set_xlim(x_min, x_max)
    ax.set_ylim(y_min, y_max)

    levels = np.linspace(np.amin(f), np.amax(f), 16, endpoint=True)

    # Contourf plot
    cfset = ax.contourf(xx, yy, f, levels, cmap='Blues')
    ## Or kernel density estimate plot instead of the contourf plot
    #ax.imshow(np.rot90(f), cmap='Blues', extent=[xmin, xmax, ymin, ymax])
    # Contour plot
    cset = ax.contour(xx, yy, f, levels, linewidths=0.5, colors='k')
    # Label plot
    # ax.clabel(cset, inline=1, fontsize=10)
    plt.xlabel('Heat conductivity spacer 1 $[$W/mK$]$')
    plt.ylabel('Heat conductivity spacer 2 $[$W/mK$]$')
    #plt.plot(345,345,'+')
    ax.set_aspect('equal')
    cbar = fig.colorbar(cfset, ticks=levels)
    cbar.set_label('pdf $[m^2K^2/W^2]$')
    plt.savefig("kde.pdf", dpi=100)


def plot_kde_1D(qoi, id, xlabel, ylabel, x_min, x_max, nx=200):

    min_qoi = x_min
    max_qoi = x_max

    kernel = compute_kde(qoi)

    qoi_lin = np.linspace(min_qoi, max_qoi, nx)
    plt.figure()
    plt.plot(qoi_lin, kernel(qoi_lin))
    plt.xlabel(xlabel)
    plt.ylabel(ylabel)
    plt.savefig('kde_' + id + '.pdf', dpi=100)


def main():

    N = 640
    base_dir = '/Users/kimliegeois/Desktop/QoI_results_27/'
    ensemble_size = 1

    N = 640
    base_dir = '/Users/kimliegeois/Desktop/QoI_results_2_AlN/'
    ensemble_size = 16

    N = 453
    base_dir = '/Users/kimliegeois/Desktop/QoI_results_3/'
    ensemble_size = 1

    N = 453
    base_dir = '/Volumes/HD_SONY/Thesis/Juelich/all_results_NER/'
    ensemble_size = 1
    data_name = 'qois_xiao_2.data'

    N = 79
    base_dir = '/Users/kimliegeois/Desktop/Juelich/order_2_30/order_20/'
    ensemble_size = 1
    data_name = 'A2_xiao_20.data'

    N = 79
    base_dir = '/Users/kimliegeois/Desktop/Juelich/ER/MPI_2/'
    ensemble_size = 1
    data_name = 'B2_xiao_20.data'

    #N = 3200
    #base_dir = '/Volumes/HD_SONY/juelich/k_conductivity/QoI_results_8/'
    #ensemble_size = 32

    n_qoi = 10
    n_rand = 3

    optical_coefficients_all = np.zeros((n_qoi, N))
    random_variables = np.zeros((n_rand, N))

    x = random_variables[0, :]
    y = random_variables[2, :]

    for i in range(0, n_rand):
        filename = base_dir + 'randomVariable_' + str(i + 1) + '.txt'
        tmp = np.loadtxt(filename)
        #print tmp[2741]
        random_variables[i, :] = tmp[0:N]
    
    N_ensemble = old_div(N, ensemble_size)
    for ensemble_index in range(0, N_ensemble):
        for l in range(0,ensemble_size):
            filename = base_dir+str(ensemble_size)+'/'+str(ensemble_index)+'/mirror_optical_results_'+str(l)+'.npz'
            npzfile = np.load(filename)
            optical_coefficients = npzfile['optical_coefficients']
            optical_coefficients_all[0:9,ensemble_index*ensemble_size+l] = compute_qoi(base_dir,ensemble_size,ensemble_index,l) #optical_coefficients     
            #optical_coefficients_all[4:9,ensemble_index*ensemble_size+l] = optical_coefficients
            optical_coefficients_all[9,ensemble_index*ensemble_size+l] = u.check_optical_tolerance(optical_coefficients_all[4:9,ensemble_index*ensemble_size+l], "mm", max_fringe = 0.0056)    
    np.save(data_name,optical_coefficients_all)
    
    optical_coefficients_all = np.load(data_name + '.npy')
    optical_coefficients_all_1 = np.copy(optical_coefficients_all)
    print(optical_coefficients_all_1)
    print(optical_coefficients_all_1.shape)
    '''
    ensemble_size = 8

    N_ensemble = N / ensemble_size
    for ensemble_index in range(0, N_ensemble):
        print ensemble_index
        for l in range(0,ensemble_size):
            filename = base_dir+str(ensemble_size)+'/'+str(ensemble_index)+'/mirror_optical_results_'+str(l)+'.npz'
            npzfile = np.load(filename)
            optical_coefficients = npzfile['optical_coefficients']
            optical_coefficients_all[:,ensemble_index*ensemble_size+l] = compute_qoi(base_dir,ensemble_size,ensemble_index,l) #optical_coefficients     

    optical_coefficients_all_8 = np.copy(optical_coefficients_all)


    print optical_coefficients_all_1
    print optical_coefficients_all_8

    print optical_coefficients_all_8-optical_coefficients_all_1
    print (optical_coefficients_all_8-optical_coefficients_all_1)/optical_coefficients_all_1
    '''

    qoi = optical_coefficients_all_1[0, :]

    kernel = compute_kde(qoi)

    T = np.linspace(100, 170, 200)
    plt.figure()
    plt.plot(T, kernel(T))
    #plt.show()

    print('-----')
    print(np.mean(qoi))
    #print np.mean(qoi[random_variables[0,:]<=random_variables[1,:]])
    #print np.mean(qoi[random_variables[0,:]>random_variables[1,:]])
    print('-----')

    mean_qoi = convergence_qoi(qoi)
    plt.figure()
    plt.plot(mean_qoi - mean_qoi[-1])

    qoi = optical_coefficients_all_1[3, :]

    print(np.min(qoi))
    print(np.max(qoi))

    kernel = compute_kde(qoi)

    max_dz = np.linspace(0.000135, 0.00015, 200)
    plt.figure()
    plt.plot(max_dz, kernel(max_dz))
    max_dz = np.linspace(0.000145, 0.00015, 100)
    ax = plt.gca()
    ax.fill_between(max_dz,
                    0,
                    kernel(max_dz),
                    facecolor='red',
                    interpolate=True)

    print('-----')
    print(np.mean(qoi))
    #print np.mean(qoi[random_variables[0,:]<=random_variables[1,:]])
    #print np.mean(qoi[random_variables[0,:]>random_variables[1,:]])
    print('-----')

    mean_qoi = convergence_qoi(qoi)
    plt.figure()
    plt.plot(mean_qoi - mean_qoi[-1])

    print(np.sum(kernel(max_dz)) * (max_dz[1] - max_dz[0]))
    #plot_kde(kernel,[370,380],[330,340],[],[0,1])
    ''' 
    ensemble_size = 1
    ensemble_index = 3
    for i in range(0, 8):
        npzfile = np.load(base_dir+str(ensemble_size)+'/'+str(ensemble_index)+'/mirror_data_' + str(i) + '.npz')
        x_i = npzfile['x']
        y_i = npzfile['y']
        z_i = 0. * x_i
        dz_i = npzfile['dz']
        tri_i = npzfile['tri']
        if i == 0:
            x = x_i
            y = y_i
            z = z_i
            dz = dz_i
            tri = tri_i
            nNodes = len(x)
        else:
            x = np.append(x, x_i, axis=0)
            y = np.append(y, y_i, axis=0)
            z = np.append(z, z_i, axis=0)
            dz = np.append(dz, dz_i, axis=0)
            tri = np.append(tri, tri_i + nNodes, axis=0)
            nNodes = len(x)             
    x_1_0 = dz[:,0]
    ensemble_size = 8
    ensemble_index = 0
    for i in range(0, 8):
        npzfile = np.load(base_dir+str(ensemble_size)+'/'+str(ensemble_index)+'/mirror_data_' + str(i) + '.npz')
        x_i = npzfile['x']
        y_i = npzfile['y']
        z_i = 0. * x_i
        dz_i = npzfile['dz']
        tri_i = npzfile['tri']
        if i == 0:
            x = x_i
            y = y_i
            z = z_i
            dz = dz_i
            tri = tri_i
            nNodes = len(x)
        else:
            x = np.append(x, x_i, axis=0)
            y = np.append(y, y_i, axis=0)
            z = np.append(z, z_i, axis=0)
            dz = np.append(dz, dz_i, axis=0)
            tri = np.append(tri, tri_i + nNodes, axis=0)
            nNodes = len(x)                        
    x_8_0 = dz[:,3] 

    plt.figure()
    plt.plot(x_1_0)
    plt.plot(x_8_0) 

    plt.figure()
    plt.plot(x_1_0-x_8_0)     
    '''

    #triang = mtri.Triangulation(x, y)

    plot_kde_2D(x, y, 50, 230, 50, 230, ngridx=200, ngridy=200)

    scales = np.linspace(0.2, 0.3, 20)

    for scale in scales:
        #150,800,150,800
        print(x.shape)
        print(y.shape)
        print(optical_coefficients_all_1.shape)
        irregularly_tol_plot(x,
                             y,
                             optical_coefficients_all_1[4:9, :],
                             50,
                             230,
                             50,
                             230,
                             ngridx=200,
                             ngridy=200,
                             scale=scale)

        plt.xlabel('Heat conductivity spacer 1 $[$W/mK$]$')
        plt.ylabel('Heat conductivity spacer 2 $[$W/mK$]$')

        #plt.plot(345,345,'+')

        scale_name = str(scale).replace(".", "_")
        plt.savefig('qoi_scale_' + scale_name + '.pdf', dpi=100)

    for i in range(0, n_qoi - 1):
        #fig = plt.figure()
        #ax = fig.gca(projection='3d')
        #ax.plot_trisurf(triang, optical_coefficients_all_1[i,:], cmap='jet')
        #irregularly_plot(x,y,optical_coefficients_all_1[i,:],150,1000,150,1000)
        #irregularly_plot(x,y,optical_coefficients_all_1[i,:],100,800,100,800)
        scale = 1
        if i == 5 or i == 6:
            scale = 1000

        fig, cbar = irregularly_plot(x,
                                     y,
                                     optical_coefficients_all_1[i, :],
                                     50,
                                     230,
                                     50,
                                     230,
                                     ngridx=2000,
                                     ngridy=2000,
                                     scale=scale)

        plt.xlabel('Heat conductivity spacer 1 $[$W/mK$]$')
        plt.ylabel('Heat conductivity spacer 2 $[$W/mK$]$')

        #plt.plot(345,345,'+')

        if i == 0:
            plt.title('Maximal temperature on the mirror surface')
            cbar.set_label('Temperature $[^\circ$C$]$')
            id_name = 'max_temp'
            xlabel = 'Temperature $[^\circ$C$]$'
            ylabel = 'pdf $[1/ \,^\circ$C$]$'
            x_min = 100
            x_max = 150
        if i == 1:
            plt.title('Minimal temperature on the mirror surface')
            cbar.set_label('Temperature $[^\circ$C$]$')
            id_name = 'min_temp'
            xlabel = 'Temperature $[^\circ$C$]$'
            ylabel = 'pdf $[1/ \,^\circ$C$]$'
            x_min = 100
            x_max = 150
        if i == 2:
            plt.title('Mean temperature on the mirror surface')
            cbar.set_label('Temperature $[^\circ$C$]$')
            id_name = 'mean_temp'
            xlabel = 'Temperature $[^\circ$C$]$'
            ylabel = 'pdf $[1/ \,^\circ$C$]$'
            x_min = 100
            x_max = 150
        if i == 3:
            plt.title('Mean z displacement of the mirror surface')
            cbar.set_label('Displacement $[$mm$]$')
            id_name = 'mean_z'
            xlabel = 'Displacement $[$mm$]$'
            ylabel = 'pdf $[1/ mm]$'
            x_min = 0.008
            x_max = 0.016
        if i == 4:
            plt.title('Translation')
            cbar.set_label('Displacement $[$mm$]$')
            id_name = 'translation'
            xlabel = 'Displacement $[$mm$]$'
            ylabel = 'pdf $[1/ mm]$'
            x_min = 0.024
            x_max = 0.03
        if i == 5:
            plt.title('Rotation y')
            cbar.set_label('$[$mrads$]$')
            id_name = 'rotation_y'
            xlabel = '$[$mrads$]$'
            ylabel = 'pdf $[1/ mrads]$'
            x_min = 0.
            x_max = 0.1
        if i == 6:
            plt.title('Rotation x')
            cbar.set_label('$[$mrads$]$')
            id_name = 'rotation_x'
            xlabel = '$[$mrads$]$'
            ylabel = 'pdf $[1/ mrads]$'
            x_min = -0.1
            x_max = 0.01
        if i == 7:
            plt.title('Curvature')
            cbar.set_label('$[$mm$]$')
            id_name = 'curvature'
            xlabel = '$[$mm$]$'
            ylabel = 'pdf $[1/ mm]$'
            x_min = -0.02
            x_max = 0.01
        if i == 8:
            plt.title('Irregularity')
            cbar.set_label('$[$mm$]$')
            id_name = 'irregularity'
            xlabel = '$[$mm$]$'
            ylabel = 'pdf $[1/ mm]$'
            x_min = 0.004
            x_max = 0.007

        plt.savefig("qoi_" + str(i) + ".pdf", dpi=100)

        plot_kde_1D(optical_coefficients_all_1[i, :] * scale,
                    id_name,
                    xlabel,
                    ylabel,
                    x_min,
                    x_max,
                    nx=1000)

        tmp = np.zeros((len(x), 3))
        tmp[:, 0] = x
        tmp[:, 1] = y
        tmp[:, 2] = optical_coefficients_all_1[i, :]

        np.savetxt("qoi_" + str(i) + ".txt", tmp)

        if i == n_qoi - 1:
            radius = 578.8807505281204
            x_circle = np.linspace(130, 560, 1000)
            #plt.plot(x_circle,np.sqrt(radius**2-x_circle**2))
        #ax.scatter(x, y, optical_coefficients_all_1[i,:])

    for i in range(0, n_qoi):
        fig = plt.figure()
        ax = fig.gca(projection='3d')

        print('-------')
        print(np.amax(optical_coefficients_all_1[i, :]))
        print(np.amin(optical_coefficients_all_1[i, :]))

        #if i == n_qoi-1:
        #    z_max = 0.000133
        #    z_min = 0.000131

        #    x_new,y_new,z_new = clip_scater(x, y, optical_coefficients_all_1[i,:],[300, 400],[300, 400],[z_min, z_max])
        #else:
        x_new, y_new, z_new = x, y, optical_coefficients_all_1[i, :]

        ax.scatter(x_new, y_new, z_new)

        #if i == n_qoi-1:
        #    ax.set_xlim(300, 400)
        #    ax.set_ylim(300, 400)
        #    ax.set_zlim(z_min, z_max)

    #fig = plt.figure()
    #ax = fig.gca(projection='3d')
    #for i in range(0, 3):
    #    ax.plot_trisurf(triang, optical_coefficients_all_1[i,:], cmap='jet')
    '''   
    fig = plt.figure()
    ax = fig.gca(projection='3d')
    ax.scatter(optical_coefficients_all[0,:], optical_coefficients_all[1,:], optical_coefficients_all[2,:])
    fig = plt.figure()
    ax = fig.gca(projection='3d')
    ax.scatter(optical_coefficients_all[0,:], optical_coefficients_all[1,:], optical_coefficients_all[3,:])
    fig = plt.figure()
    ax = fig.gca(projection='3d')    
    ax.scatter(optical_coefficients_all[0,:], optical_coefficients_all[1,:], optical_coefficients_all[4,:])
    '''

    #kernel = compute_kde(optical_coefficients_all)

    #plot_kde(kernel,[0.0915,0.94],[1.78638639e-04,2.01983639e-04],[-5.16471018e-05,-6.05417325e-02,4.14012494e-02],[0,1])

    qoi = optical_coefficients_all_1[-1, :]

    not_blurred = np.sum(qoi) / float(len(qoi))
    blurred = 1. - not_blurred

    print('not blurred')
    print(not_blurred)
    print('blurred')
    print(blurred)

    plt.show()


if __name__ == "__main__":
    main()
