#!/usr/bin/env python
# -*- coding: latin-1; -*-

from __future__ import print_function
from __future__ import division
from builtins import str
from builtins import range
from past.utils import old_div
from katropton.speedup import *
import matplotlib.pyplot as plt
import os


def plot_iter_all_ensemble_sizes(n_samples, iter):
    import matplotlib.pyplot as plt
    plt.figure()
    plt.plot(list(range(0, n_samples)),
             iter[0][:], 'bo', markersize=8, zorder=1)
    if iter.shape[0] >= 2:
        plt.plot(list(range(0, n_samples)),
                 iter[1][:], 'ro', markersize=6, zorder=2)
    if iter.shape[0] >= 3:
        plt.plot(list(range(0, n_samples)),
                 iter[2][:], 'go', markersize=4, zorder=2)
    if iter.shape[0] >= 4:
        plt.plot(list(range(0, n_samples)),
                 iter[3][:], 'ko', markersize=2, zorder=3)
    plt.xlabel('sample')
    plt.ylabel('Number of iterations to converge')


def speedup_of_one_case(file_MC_base_dir,
                        n_samples,
                        ensemble_sizes,
                        quantiles,
                        querylines,
                        use_waves_timers,
                        use_teuchos_timers,
                        per_iteration,
                        iter_2,
                        reduction=False):
    import matplotlib.pyplot as plt

    if reduction:
        name = 'case_ER'
    else:
        name = 'case'

    speedup = compute_speedup(n_samples,
                              ensemble_sizes,
                              file_MC_base_dir,
                              querylines,
                              per_iteration,
                              quantiles,
                              use_waves_timers,
                              use_teuchos_timers,
                              reduction,
                              remove_write=True,
                              remove_block=True)

    plot_speedup(speedup, ensemble_sizes, np.arange(0, len(querylines)), name,
                 querylines, per_iteration)
    plt.savefig(name + '.png', dpi=400)

    save_speedup(speedup, ensemble_sizes, querylines, name)

    R = compute_R(n_samples, ensemble_sizes, file_MC_base_dir, quantiles,
                  reduction)

    save_R(R, name)

    iter = read_iter_all_ensemble_sizes(n_samples, ensemble_sizes,
                                        file_MC_base_dir, reduction)

    # if not reduction:
    #    iter[0][:] = iter_2[0][:]
    if len(ensemble_sizes) == 4:
        plot_iter_all_ensemble_sizes(n_samples, iter)
        plt.savefig(name + '_convergence.png', dpi=400)
        np.savetxt(name + '_convergence.txt', iter, delimiter=' ')

    return iter


def write_tex(n_samples, iter, filename):
    f = open(filename, 'w')
    print('\\addplot [c1, mark=*, mark size=2pt, mark options={solid}, only marks]', file=f)
    print('table [row sep=crcr] {%', file=f)
    for i in range(0, n_samples):
        print(str(i) + ' ' + str(iter[0][i]) + '\\\\', file=f)
    print('};', file=f)
    print('\\label{pgfplots:noEP}', file=f)
    if iter.shape[0] >= 2:
        ensemble_size = 8
        for i in range(0, int(np.ceil(n_samples * 1. / ensemble_size))):
            print('\\addplot [line width=2pt, c2]', file=f)
            print('table [row sep=crcr] {%', file=f)
            print(str(ensemble_size * i) + ' ' + str(
                iter[1][ensemble_size * i]) + '\\\\', file=f)
            print(str(ensemble_size *
                      (i + 1)) + ' ' + str(iter[1][ensemble_size *
                                                   (i + 1) - 1]) + '\\\\', file=f)
            print('};', file=f)
        print('\\label{pgfplots:s8}', file=f)
    if iter.shape[0] >= 3:
        ensemble_size = 16
        for i in range(0, int(np.ceil(n_samples * 1. / ensemble_size))):
            print('\\addplot [line width=1.5pt, c3]', file=f)
            print('table [row sep=crcr] {%', file=f)
            print(str(ensemble_size * i) + ' ' + str(
                iter[2][ensemble_size * i]) + '\\\\', file=f)
            print(str(ensemble_size *
                      (i + 1)) + ' ' + str(iter[2][ensemble_size *
                                                   (i + 1) - 1]) + '\\\\', file=f)
            print('};', file=f)
        print('\\label{pgfplots:s16}', file=f)
    if iter.shape[0] >= 4:
        ensemble_size = 32
        for i in range(0, int(np.ceil(n_samples * 1. / ensemble_size))):
            print('\\addplot [line width=1.pt, c4]', file=f)
            print('table [row sep=crcr] {%', file=f)
            print(str(ensemble_size * i) + ' ' + str(
                iter[3][ensemble_size * i]) + '\\\\', file=f)
            print(str(ensemble_size *
                      (i + 1)) + ' ' + str(iter[3][ensemble_size *
                                                   (i + 1) - 1]) + '\\\\', file=f)
            print('};', file=f)
        print('\\label{pgfplots:s32}', file=f)
    f.closed


def plot_iter_all(base_dir_ER, base_dir_NER, ensemble_sizes, n_samples):

    iter_NER = read_iter_all_ensemble_sizes(n_samples, ensemble_sizes,
                                            base_dir_NER, False)
    iter_ER = read_iter_all_ensemble_sizes(n_samples, ensemble_sizes,
                                           base_dir_ER, True)
    plot_iter_all_ensemble_sizes(n_samples, iter_NER)
    write_tex(n_samples, iter_NER, 'NER_conv.tex')
    np.savetxt('NER_conv.txt', iter_NER)
    np.savetxt('ordering_1.txt',
               np.argsort(iter_NER[0, :]).astype(int),
               fmt='%i')
    np.savetxt('ordering_2.txt',
               np.argsort(iter_NER[0][:]).astype(int),
               fmt='%i')
    plot_iter_all_ensemble_sizes(n_samples, iter_ER)
    write_tex(n_samples, iter_ER, 'ER_conv.tex')
    np.savetxt('ER_conv.txt', iter_ER)

    order = np.argsort(iter_NER[0, :])

    tmp = iter_NER[0][order]

    plt.figure()
    plt.plot(iter_NER[0][:])
    plt.savefig('no_order_1.png', dpi=400)

    iter_NER[0][:] = tmp

    plt.figure()
    plt.plot(tmp)
    plt.savefig('order_1.png', dpi=400)

    tmp = iter_ER[0][order]

    plt.figure()
    plt.plot(iter_ER[0][:])
    plt.savefig('no_order_2.png', dpi=400)

    iter_ER[0][:] = tmp

    write_tex(n_samples, iter_NER, 'NER_conv_2.tex')
    write_tex(n_samples, iter_ER, 'ER_conv_2.tex')

    plt.figure()
    plt.plot(tmp)
    plt.savefig('order_2.png', dpi=400)


def CPU_time_per_one_case(base_dir,
                          n_samples,
                          ensemble_sizes,
                          quantiles,
                          querylines,
                          use_waves_timers,
                          use_teuchos_timers,
                          per_iteration,
                          filename_CPU,
                          filename_speed_up,
                          total_id,
                          write_id,
                          block_id,
                          per_ensemble=True):

    timers = read_timers_all_ensemble_sizes(n_samples, ensemble_sizes,
                                            base_dir, querylines,
                                            use_waves_timers,
                                            use_teuchos_timers)

    f = open(filename_CPU, 'w')
    print(timers.shape, file=f)

    for i in range(0, timers.shape[2]):
        print(querylines[i], file=f)
        string = ''
        for e in range(0, timers.shape[0]):
            if per_ensemble:
                current_timer = old_div(np.sum(timers[e, :, i]), n_samples)
                if i == total_id:
                    current_timer -= old_div(
                        np.sum(timers[e, :, write_id]), n_samples)
                    current_timer -= old_div(
                        np.sum(timers[e, :, block_id]), n_samples)

            else:
                current_timer = old_div(np.sum(
                    timers[e, :, i]), (n_samples * ensemble_sizes[e]))
                if i == total_id:
                    current_timer -= old_div(np.sum(timers[e, :, write_id]), (
                        n_samples * ensemble_sizes[e]))
                    current_timer -= old_div(np.sum(timers[e, :, block_id]), (
                        n_samples * ensemble_sizes[e]))

            string = string + str(round(current_timer, 3)) + ' & '
        print(string, file=f)
        print(' ', file=f)
    f.closed

    f = open(filename_speed_up, 'w')
    print(timers.shape, file=f)

    for i in range(0, timers.shape[2]):
        print(querylines[i], file=f)
        string = ''
        for e in range(0, timers.shape[0]):
            if e == 0:
                current_timer_s1 = np.sum(timers[0, :, i])
                if i == total_id:
                    current_timer_s1 -= np.sum(timers[0, :, write_id])
                    current_timer_s1 -= np.sum(timers[0, :, block_id])

            current_timer = old_div(np.sum(timers[e, :, i]), ensemble_sizes[e])
            if i == total_id:
                current_timer -= old_div(np.sum(
                    timers[e, :, write_id]), ensemble_sizes[e])
                current_timer -= old_div(np.sum(
                    timers[e, :, block_id]), ensemble_sizes[e])

            string = string + str(round(old_div(current_timer_s1, current_timer),
                                        3)) + ' & '
        print(string, file=f)
        print(' ', file=f)
    f.closed


def analysis_one_case(directory,
                      base_dir,
                      n_samples,
                      ensemble_sizes,
                      quantiles,
                      querylines,
                      use_waves_timers,
                      use_teuchos_timers,
                      per_iteration,
                      ensemble_reduction=False):
    if not os.path.exists(directory):
        os.makedirs(directory)
    os.chdir(directory)

    iter = speedup_of_one_case(base_dir, n_samples, ensemble_sizes, quantiles,
                               querylines, use_waves_timers,
                               use_teuchos_timers, per_iteration, np.array([]),
                               ensemble_reduction)

    CPU_time_per_one_case(base_dir, n_samples, ensemble_sizes, quantiles,
                          querylines, use_waves_timers, use_teuchos_timers,
                          per_iteration, 'average_CPU.txt',
                          'average_speed_up.txt', 7, 8, 21)

    #plot_iter_all(base_dir_ER, base_dir_NER, ensemble_sizes, n_samples)
    iters = read_iter_all_ensemble_sizes(n_samples, ensemble_sizes, base_dir,
                                         ensemble_reduction)
    plot_iter_all_ensemble_sizes(n_samples, iters)
    write_tex(n_samples, iters, 'conv.tex')

    os.chdir('..')


def main():
    import matplotlib.pyplot as plt

    ensemble_sizes = [1, 8, 16, 32]
    quantiles = [0, 0.33, 0.5, 0.66, 1.]

    querylines = [
        'computeMatrices', 'Belos: Operation Op*x', 'Belos: Operation Prec*x',
        'Belos: DGKS[2]: Orthogonalization',
        'Belos: DGKS[2]: Ortho (Inner Product)',
        'Belos: DGKS[2]: Ortho (Norm)', 'Belos: DGKS[2]: Ortho (Update)',
        'total', 'write results', 'MueLu: Hierarchy: Solve (level=0)',
        'MueLu: Hierarchy: Solve (level=1)',
        'MueLu: Hierarchy: Solve (level=2)',
        'MueLu: Hierarchy: Solve : smoothing (level=0)',
        'MueLu: Hierarchy: Solve : prolongation (level=0)',
        'MueLu: Hierarchy: Solve : residual calculation (level=0)',
        'MueLu: Hierarchy: Solve : restriction (level=0)',
        'MueLu: Hierarchy: Solve : smoothing (level=0)',
        'MueLu: Hierarchy: Solve : smoothing (level=1)',
        'MueLu: Hierarchy: Solve : coarse (level=2)',
        'Belos: PseudoBlockGmresSolMgr total solve time',
        'MueLu: Hierarchy: Setup (total)', 'create the block matrix'
    ]
    use_waves_timers = np.array([0, 7, 8, 21], dtype=int)
    use_teuchos_timers = np.array(
        [1, 2, 3, 4, 5, 6, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20],
        dtype=int)
    per_iteration = [
        False, True, True, False, False, False, False, False, False, True,
        True, True, True, True, True, True, True, True, True, False, False,
        False
    ]

    n_samples = 64

    ensemble_sizes = [1, 8, 16]

    base_dir = '/Users/kimliegeois/Desktop/Juelich/ER/mpi_ER/MPI_2'

    directory = 'option_A_mpi_2_ER'
    analysis_one_case(directory, base_dir, n_samples, ensemble_sizes,
                      quantiles, querylines, use_waves_timers,
                      use_teuchos_timers, per_iteration, True)

    base_dir = '/Users/kimliegeois/Desktop/Juelich/NER/mpi_NER/MPI_2'

    directory = 'option_A_mpi_2_NER'
    analysis_one_case(directory, base_dir, n_samples, ensemble_sizes,
                      quantiles, querylines, use_waves_timers,
                      use_teuchos_timers, per_iteration, False)

    ensemble_sizes = [1, 8, 16, 32]

    base_dir = '/Users/kimliegeois/Desktop/Juelich/ER/mpi_ER/MPI_4'

    directory = 'option_A_mpi_4_ER'
    analysis_one_case(directory, base_dir, n_samples, ensemble_sizes,
                      quantiles, querylines, use_waves_timers,
                      use_teuchos_timers, per_iteration, True)

    base_dir = '/Users/kimliegeois/Desktop/Juelich/NER/mpi_NER/MPI_4'

    directory = 'option_A_mpi_4_NER'
    analysis_one_case(directory, base_dir, n_samples, ensemble_sizes,
                      quantiles, querylines, use_waves_timers,
                      use_teuchos_timers, per_iteration, False)

    base_dir = '/Users/kimliegeois/Desktop/Juelich/ER/mpi_ER/MPI_8'

    directory = 'option_A_mpi_8_ER'
    analysis_one_case(directory, base_dir, n_samples, ensemble_sizes,
                      quantiles, querylines, use_waves_timers,
                      use_teuchos_timers, per_iteration, True)

    base_dir = '/Users/kimliegeois/Desktop/Juelich/NER/mpi_NER/MPI_8'

    directory = 'option_A_mpi_8_NER'
    analysis_one_case(directory, base_dir, n_samples, ensemble_sizes,
                      quantiles, querylines, use_waves_timers,
                      use_teuchos_timers, per_iteration, False)

    n_samples = 32
    ensemble_sizes = [1, 8, 16]
    base_dir = '/Users/kimliegeois/Desktop/Juelich/mpi_4/MPI_16'

    directory = 'option_A_mpi_16_all'
    print(directory)
    analysis_one_case(directory, base_dir, n_samples, ensemble_sizes,
                      quantiles, querylines, use_waves_timers,
                      use_teuchos_timers, per_iteration, False)

    n_samples = 32
    ensemble_sizes = [1, 8, 16, 32]
    base_dir = '/Users/kimliegeois/Desktop/Juelich/mpi_4/MPI_32'

    directory = 'option_A_mpi_32_all'
    print(directory)
    analysis_one_case(directory, base_dir, n_samples, ensemble_sizes,
                      quantiles, querylines, use_waves_timers,
                      use_teuchos_timers, per_iteration, False)

    n_samples = 16
    ensemble_sizes = [1, 8, 16]
    base_dir = '/Users/kimliegeois/Desktop/Juelich/mpi_5/MPI_32'

    directory = 'option_A_mpi_32_all_2'
    print(directory)
    analysis_one_case(directory, base_dir, n_samples, ensemble_sizes,
                      quantiles, querylines, use_waves_timers,
                      use_teuchos_timers, per_iteration, False)

    n_samples = 32
    ensemble_sizes = [1, 8, 16, 32]
    base_dir = '/Users/kimliegeois/Desktop/Juelich/mpi_6/MPI_32'

    directory = 'option_A_mpi_32_all_3'
    print(directory)
    analysis_one_case(directory, base_dir, n_samples, ensemble_sizes,
                      quantiles, querylines, use_waves_timers,
                      use_teuchos_timers, per_iteration, False)


if __name__ == "__main__":
    main()
