#!/bin/bash
# Submission script for Blake
#SBATCH --job-name=Mirror
#SBATCH --time=41:00:00 # hh:mm:ss
#
#SBATCH -N 1
#SBATCH -p blake
#
#SBATCH --comment=test
#SBATCH --output=workspace/Mirror.%j.out

. ~/.bash_profile $SLURM_SUBMIT_HOST

export n_nodes=1

function setEnsembleReduction(){
  remove2env INCLUDE "${Trilinos_DIR}/include"
  remove2env LIB "${Trilinos_DIR}/lib"
  remove2env PYTHONPATH "${Trilinos_DIR}/lib/python2.7/site-packages"
  remove2env LD_LIBRARY_PATH "${Trilinos_DIR}/lib/python2.7/site-packages/PyTrilinos"
  remove2env LD_LIBRARY_PATH "${Trilinos_DIR}/lib/python2.7/site-packages"
  remove2env LD_LIBRARY_PATH "${Trilinos_DIR}/lib"
  if [ “$1” == “false” ]; then
    export Trilinos_DIR=$Trilinos_DIR_NO_ER
    export Waves_DIR=$Waves_DIR_NO_ER
    export ensemble_reduction="false"
  else
    export Trilinos_DIR=$Trilinos_DIR_ER
    export Waves_DIR=$Waves_DIR_ER
    export ensemble_reduction="true"
  fi
  add2env INCLUDE "${Trilinos_DIR}/include" end
  add2env LIB "${Trilinos_DIR}/lib" end
  add2env PYTHONPATH "${Trilinos_DIR}/lib/python2.7/site-packages" front
  add2env LD_LIBRARY_PATH "${Trilinos_DIR}/lib/python2.7/site-packages/PyTrilinos" end
  add2env LD_LIBRARY_PATH "${Trilinos_DIR}/lib/python2.7/site-packages" end
  add2env LD_LIBRARY_PATH "${Trilinos_DIR}/lib" end
  cd $Waves_DIR
}

export MKL_DYNAMIC=TRUE
export OMP_DYNAMIC=FALSE
export OMP_NUM_THREADS=24
export KMP_PLACE_THREADS=24c,1t
export KMP_AFFINITY=compact

export MKL_NUM_THREADS=${OMP_NUM_THREADS}
export N_MPI_PER_SOCKET=1

ensemble_sizes=( 1 8 16 32)
run_NER=true
run_ER=true

setEnsembleReduction false

n_MPI=$(( 2*n_nodes ))
base_dir=$(pwd)
msh_file=${base_dir}/katropton/Juelich/mesh/fused_mirror_assembly_mm.msh
workspace_NER_dir=${base_dir}/workspace/katropton_Juelich_tests_M1_assembly_fused_MC_k
msh_part_NER_file=${workspace_NER_dir}/fused_mirror_assembly_mm_part_${n_MPI}.msh

if [ ! -d ${workspace_NER_dir} ] 
then
    mkdir ${workspace_NER_dir}
fi

if [ ! -e ${msh_part_NER_file} ]
then
    gmsh -3 ${msh_file} -o ${msh_part_NER_file} -part ${n_MPI}
fi

if [ "$run_NER" = true ]; then
  for s in "${ensemble_sizes[@]}"
  do
    export ensemble_size=$s
    mpirun -npersocket ${N_MPI_PER_SOCKET} python run.py -k ${OMP_NUM_THREADS} katropton/Juelich/tests/M1_assembly_fused_MC_k.py
  done
fi

ensemble_sizes=( 8 16 32)

setEnsembleReduction true

base_dir=$(pwd)
workspace_ER_dir=${base_dir}/workspace/katropton_Juelich_tests_M1_assembly_fused_MC_k
msh_part_ER_file=${workspace_ER_dir}/fused_mirror_assembly_mm_part_${n_MPI}.msh

if [ ! -d ${workspace_ER_dir} ] 
then
    mkdir ${workspace_ER_dir}
fi

cp ${msh_part_NER_file} ${msh_part_ER_file}

if [ "$run_ER" = true ]; then
  for s in "${ensemble_sizes[@]}"
  do
    export ensemble_size=$s
    mpirun -npersocket ${N_MPI_PER_SOCKET} python run.py -k ${OMP_NUM_THREADS} katropton/Juelich/tests/M1_assembly_fused_MC_k.py
  done
fi