#ifndef KATROPTON_MATERIAL_H
#define KATROPTON_MATERIAL_H

#include "katropton.h"

#include "Map.h"
#include "wProblem.h"
#include "wTag.h"
#include "wMshData.h"
#include "wElement.h"
#include "wNode.h"
#include "wMedium.h"

#include <Tpetra_Map.hpp>
#include <Tpetra_Vector.hpp>
#include <Teuchos_RCP.hpp>
#include <Kokkos_ViewFactory.hpp>

namespace katropton
{

/**
 * @brief Class which includes all constitutive values of a given material.
 */
template <typename scalar>
class Material
{
private:
    scalar E;
    scalar nu;
    scalar k;
    scalar beta;

public:
    Material(scalar _E, scalar _nu, scalar _k, scalar _beta);

    /**
    * @brief Return \f$E\f$
    */
    inline scalar getE() { return E; }

    /**
    * @brief Return \f$\nu\f$
    */
    inline scalar getNu() { return nu; }

    /**
    * @brief Return \f$k\f$
    */
    inline scalar getK() { return k; }

    /**
    * @brief Return \f$\beta\f$
    */
    inline scalar getBeta() { return beta; }
};

/**
 * @brief Material constructor
 * 
 * This constructor creates a new katropton::Material which store the constitutive parameters.
 * 
 * Arguments:
 * <ul>
 *  <li> _E: the Young modulus \f$E\f$,
 *  <li> _nu: the Poisson coefficient \f$\nu\f$,
 *  <li> _k: the heat conductivity \f$k\f$,
 *  <li> _beta: the coefficient of thermal expansion \f$\beta\f$.
 * </ul>
 */
template <typename scalar>
Material<scalar>::Material(scalar _E, scalar _nu, scalar _k, scalar _beta) : E(_E), nu(_nu), k(_k), beta(_beta)
{
}
}; // namespace katropton

#endif //KATROPTON_MATERIAL_H
