#!/usr/bin/env python
# -*- coding: latin-1; -*-

import katropton as m
import tbox
import tbox.gmsh as gmsh
import katropton.utilities as u
from fwk.testing import *
import os


def test(msh, xml_file_name, rank, nThreads):

    file_dir = os.path.dirname(__file__)
    work_dir = os.getcwd()

    ensemble_size = 1

    pbl = m.Problem(msh, comm)

    E = 1. * np.ones(ensemble_size)
    nu = 0.2 * np.ones(ensemble_size)
    k = 0.114
    d = 0.

    hf = 1 * np.ones(ensemble_size)
    zero = np.zeros(ensemble_size)
    m.Medium(pbl, "Body 1", "test", E, nu, k, d)

    m.Dirichlet(pbl, "Surf 1 1", "Clamped", 1, 0., 1, 0., 1, 0., 1, 10.,
                ensemble_size)
    m.Neumann(pbl, "Surf 1 2", "heat fluxes", 0, zero, 0, zero, 0, zero, 1, hf,
              ensemble_size)

    m.Weight(pbl, "Surf 1 2", 1, 1., 1, 1., 1, 1., 1, 2.5, ensemble_size)

    dx = np.zeros(ensemble_size)
    dy = 2. * np.ones(ensemble_size)
    dz = np.zeros(ensemble_size)
    dT = 10. * np.ones(ensemble_size)

    solverList = Teuchos.ParameterList()
    solverList['Ensemble Convergence Tolerance'] = 10**(-4)

    solverList['Maximum Iterations'] = 1000
    solverList['Verbosity'] = 33
    solverList['Flexible Gmres'] = False
    solverList['Output Style'] = 1
    solverList['Output Frequency'] = 1

    solverList['Use blocked matrix'] = True
    solverList['Use blocked status test'] = True
    solverList['Use blocked status test: x'] = True
    solverList['Use blocked status test: T'] = True
    solverList['Ensemble Convergence Tolerance: x'] = 10**(-7)
    solverList['Ensemble Convergence Tolerance: T'] = 10**(-7)
    solverList['Use weighted status test'] = True
    solverList['Ensemble Convergence Tolerance: weights'] = 10**(-7)

    solverList['Write txt files'] = True
    solverList['Write matrix and vectors'] = True

    solverList["use xml file"] = True
    solverList[
        "MueLu xml file name"] = file_dir + '/../preconditioners/' + xml_file_name
    solverList["convert MueLu xml file"] = True

    solverList['Use preconditioner'] = True

    slv = m.IterativeSolver(pbl, solverList, 4, ensemble_size)

    slv.start()

    tag_name = "Measure point 1"
    which_pos = 0
    which_dof = 3
    DOFperNode = 4
    name_sol = "x_mm.txt"

    pos_1, sol_1 = u.extract(msh, tag_name, DOFperNode, which_pos, which_dof,
                             ensemble_size, name_sol)

    tag_name = "Measure point 2"

    pos_2, sol_2 = u.extract(msh, tag_name, DOFperNode, which_pos, which_dof,
                             ensemble_size, name_sol)

    tests = CTests()
    measured_T_1 = sol_1[0]
    measured_T_2 = sol_2[0]
    tests.add(CTest('Temperature 1', measured_T_1, 97.7193, 5e-5))
    tests.add(CTest('Temperature 2', measured_T_2, 10., 5e-5))
    tests.run()


if __name__ == "__main__":
    comm, rank, size = u.import_MPI()

    from PyTrilinos import Teuchos

    import shlex
    import subprocess
    import numpy as np

    file_dir = os.path.dirname(__file__)
    work_dir = os.getcwd()

    geo_name = '1_cube_tet.geo'
    file_dir = os.path.dirname(__file__)
    work_dir = os.getcwd()

    msh, mesh_name, partitioned_mesh_name = u.read_mesh_MPI(
        geo_name, file_dir, work_dir, comm, rank, size)

    nThreads = u.Initialize_Kokkos()
    directory = '1'
    u.mkdir_MPI(directory, comm, rank, size)
    os.chdir(directory)
    test(msh, 'AMG_BGS_gs_gs.xml', rank, nThreads)
    os.chdir('..')

    directory = '2'
    u.mkdir_MPI(directory, comm, rank, size)
    os.chdir(directory)
    test(msh, 'blockdirect.xml', rank, nThreads)

    u.Finalize_Kokkos()
