#!/usr/bin/env python
# -*- coding: latin-1; -*-

from __future__ import print_function
from __future__ import division
from builtins import str
from builtins import range
from past.utils import old_div
import katropton as m
import tbox
import tbox.gmsh as gmsh
import katropton.utilities as u
import katropton.eigenvalues as e
import katropton.readers as r
import katropton.convergence as c
from PyTrilinos import Teuchos
import shlex
import subprocess
import os
import numpy as np
import tarfile
import glob
from fwk.wutils import parseargs
import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import axes3d
from katropton.speedup import *


def read_mm_vector(filename, ensemble_size):
    with open(filename, 'r') as my_file:
        text = my_file.read().splitlines(True)
        if ensemble_size > 1:
            for i in range(2, len(text)):
                text[i] = text[i].replace("[", "")
                text[i] = text[i].replace("]", "")

    X = np.genfromtxt(text[2:])
    return X


def extract_values(filename, node_id, which_dof, dof_per_node, ensemble_size):
    output = np.zeros((ensemble_size, ))
    X = read_mm_vector(filename, ensemble_size)
    output = X[dof_per_node * node_id + which_dof, :]
    return output


def write_tex(n_samples, iter, filename):
    f = open(filename, 'w')
    print('\\addplot [c1, mark=*, mark size=2pt, mark options={solid}, only marks]', file=f)
    print('table [row sep=crcr] {%', file=f)
    for i in range(0, n_samples):
        print(str(i) + ' ' + str(iter[0][i]) + '\\\\', file=f)
    print('};', file=f)
    print('\\label{pgfplots:noEP}', file=f)
    ensemble_size = 8
    for i in range(0, int(np.ceil(n_samples * 1. / ensemble_size))):
        print('\\addplot [line width=2pt, c2]', file=f)
        print('table [row sep=crcr] {%', file=f)
        print(str(ensemble_size * i) + ' ' + str(
            iter[1][ensemble_size * i]) + '\\\\', file=f)
        print(str(ensemble_size *
                  (i + 1)) + ' ' + str(iter[1][ensemble_size *
                                               (i + 1) - 1]) + '\\\\', file=f)
        print('};', file=f)
    print('\\label{pgfplots:s8}', file=f)
    ensemble_size = 16
    for i in range(0, int(np.ceil(n_samples * 1. / ensemble_size))):
        print('\\addplot [line width=1.5pt, c3]', file=f)
        print('table [row sep=crcr] {%', file=f)
        print(str(ensemble_size * i) + ' ' + str(
            iter[2][ensemble_size * i]) + '\\\\', file=f)
        print(str(ensemble_size *
                  (i + 1)) + ' ' + str(iter[2][ensemble_size *
                                               (i + 1) - 1]) + '\\\\', file=f)
        print('};', file=f)
    print('\\label{pgfplots:s16}', file=f)
    ensemble_size = 32
    for i in range(0, int(np.ceil(n_samples * 1. / ensemble_size))):
        print('\\addplot [line width=1.pt, c4]', file=f)
        print('table [row sep=crcr] {%', file=f)
        print(str(ensemble_size * i) + ' ' + str(
            iter[3][ensemble_size * i]) + '\\\\', file=f)
        print(str(ensemble_size *
                  (i + 1)) + ' ' + str(iter[3][ensemble_size *
                                               (i + 1) - 1]) + '\\\\', file=f)
        print('};', file=f)
    print('\\label{pgfplots:s32}', file=f)
    f.closed


def CPU_time_per_one_case(base_dir,
                          n_samples,
                          ensemble_sizes,
                          quantiles,
                          querylines,
                          use_waves_timers,
                          use_teuchos_timers,
                          per_iteration,
                          filename_CPU,
                          filename_speed_up,
                          per_ensemble=True):

    timers = read_timers_all_ensemble_sizes(n_samples, ensemble_sizes,
                                            base_dir, querylines,
                                            use_waves_timers,
                                            use_teuchos_timers)

    f = open(filename_CPU, 'w')
    print(timers.shape, file=f)

    for i in range(0, timers.shape[2]):
        print(querylines[i], file=f)
        string = ''
        for e in range(0, timers.shape[0]):
            if per_ensemble:
                current_timer = old_div(np.sum(timers[e, :, i]), n_samples)
            else:
                current_timer = old_div(np.sum(
                    timers[e, :, i]), (n_samples * ensemble_sizes[e]))
            string = string + str(round(current_timer, 3)) + ' & '
        print(string, file=f)
        print(' ', file=f)
    f.closed

    f = open(filename_speed_up, 'w')
    print(timers.shape, file=f)

    for i in range(0, timers.shape[2]):
        print(querylines[i], file=f)
        string = ''
        for e in range(0, timers.shape[0]):
            if e == 0:
                current_timer_s1 = np.sum(timers[0, :, i])
            current_timer = old_div(np.sum(timers[e, :, i]), ensemble_sizes[e])
            string = string + str(round(old_div(current_timer_s1, current_timer),
                                        3)) + ' & '
        print(string, file=f)
        print(' ', file=f)
    f.closed


def load_qoi(base_dir, N, ensemble_size):
    node_id = 68
    #node_id = 227
    which_dof = 2
    dof_per_node = 3

    qoi = np.zeros((N, ))
    N_ensemble = old_div(N, ensemble_size)

    ensemble_index_min = 0
    for ensemble_index in range(ensemble_index_min, N_ensemble):
        solution_file_name = base_dir + '/' + str(ensemble_size) + '/' + str(
            ensemble_index) + '/x_mm.txt'
        output = extract_values(solution_file_name, node_id, which_dof,
                                dof_per_node, ensemble_size)

        ensemble_index_start = ensemble_index * ensemble_size
        ensemble_index_end = ensemble_index_start + ensemble_size

        qoi[ensemble_index_start:ensemble_index_end] = output

    return qoi


def speedup(base_dir,
            n_samples,
            ensemble_sizes,
            quantiles,
            querylines,
            use_waves_timers,
            use_teuchos_timers,
            per_iteration,
            ensemble_reduction=False):
    import matplotlib.pyplot as plt

    if ensemble_reduction:
        case_id = 'ER'
    else:
        case_id = 'NER'

    speedup = compute_speedup(n_samples,
                              ensemble_sizes,
                              base_dir,
                              querylines,
                              per_iteration,
                              quantiles,
                              use_waves_timers,
                              use_teuchos_timers,
                              reduction=ensemble_reduction,
                              remove_write=True)

    plot_speedup(speedup, ensemble_sizes, np.arange(0, len(querylines)),
                 'beam_' + case_id, querylines, per_iteration)
    plt.savefig('beam_' + case_id + '.png', dpi=400)

    save_speedup(speedup, ensemble_sizes, querylines, 'beam_' + case_id)

    R = compute_R(n_samples, ensemble_sizes, base_dir, quantiles,
                  ensemble_reduction)
    save_R(R, 'beam_R_' + case_id)


def speedup_ensemble_reduction(base_dir_ER, base_dir_NER, n_samples):
    ensemble_sizes = [1, 8, 16, 32]
    quantiles = [0, 0.33, 0.5, 0.66, 1.]

    querylines = [
        'computeMatrices', 'Belos: Operation Op*x', 'Belos: Operation Prec*x',
        'Belos: DGKS[2]: Orthogonalization',
        'Belos: DGKS[2]: Ortho (Inner Product)',
        'Belos: DGKS[2]: Ortho (Norm)', 'Belos: DGKS[2]: Ortho (Update)',
        'total', 'write results', 'MueLu: Hierarchy: Solve (level=0)',
        'MueLu: Hierarchy: Solve (level=1)',
        'MueLu: Hierarchy: Solve : smoothing (level=0)',
        'MueLu: Hierarchy: Solve : prolongation (level=0)',
        'MueLu: Hierarchy: Solve : residual calculation (level=0)',
        'MueLu: Hierarchy: Solve : restriction (level=0)',
        'MueLu: SIMPLE: Apply: Update step - level 0',
        'MueLu: SIMPLE: Apply: Prediction step: Solve block 00 - level 0',
        'MueLu: SIMPLE: Apply: Prediction step: Compute RHS - level 0',
        'MueLu: SIMPLE: Apply: Correction step: Solve block 11 - level 0',
        'MueLu: SIMPLE: Apply: Correction step: Compute RHS - level 0'
    ]
    use_waves_timers = np.array([0, 7, 8], dtype=int)
    use_teuchos_timers = np.array(
        [1, 2, 3, 4, 5, 6, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19],
        dtype=int)
    per_iteration = [
        False, True, True, False, False, False, False, False, False, True,
        True, True, True, True, True, True, True, True, True, True
    ]

    speedup(base_dir_NER, n_samples, ensemble_sizes, quantiles, querylines,
            use_waves_timers, use_teuchos_timers, per_iteration, False)
    speedup(base_dir_ER, n_samples, ensemble_sizes, quantiles, querylines,
            use_waves_timers, use_teuchos_timers, per_iteration, True)

    querylines = [
        'domain', 'Belos: Operation Op*x', 'Belos: Operation Prec*x',
        'Belos: DGKS[2]: Orthogonalization',
        'Belos: PseudoBlockGmresSolMgr total solve time',
        'MueLu: Hierarchy: Setup (total)', 'Belos: DGKS[2]: Ortho (Update)',
        'total', 'write results', 'MueLu: Hierarchy: Solve (level=0)',
        'MueLu: Hierarchy: Solve (level=1)',
        'MueLu: Hierarchy: Solve : smoothing (level=0)',
        'MueLu: Hierarchy: Solve : prolongation (level=0)',
        'MueLu: Hierarchy: Solve : residual calculation (level=0)',
        'MueLu: Hierarchy: Solve : restriction (level=0)',
        'MueLu: SIMPLE: Apply: Update step - level 0',
        'MueLu: SIMPLE: Apply: Prediction step: Solve block 00 - level 0',
        'MueLu: SIMPLE: Apply: Prediction step: Compute RHS - level 0',
        'MueLu: SIMPLE: Apply: Correction step: Solve block 11 - level 0',
        'MueLu: SIMPLE: Apply: Correction step: Compute RHS - level 0',
        'compute contact matrices', 'compute block 00 matrix', 'dp',
        'computeMatrices'
    ]
    use_waves_timers = np.array([0, 7, 8, 20, 21, 22, 23], dtype=int)
    use_teuchos_timers = np.array(
        [1, 2, 3, 4, 5, 6, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19],
        dtype=int)
    per_iteration = [
        False, True, True, False, False, False, False, False, False, True,
        True, True, True, True, True, True, True, True, True, True, False
    ]

    CPU_time_per_one_case(base_dir_NER, n_samples, ensemble_sizes, quantiles,
                          querylines, use_waves_timers, use_teuchos_timers,
                          per_iteration, 'average_CPU_NER.txt',
                          'average_speed_up_NER.txt')

    CPU_time_per_one_case(base_dir_ER, n_samples, ensemble_sizes, quantiles,
                          querylines, use_waves_timers, use_teuchos_timers,
                          per_iteration, 'average_CPU_ER.txt',
                          'average_speed_up_ER.txt')


def plot_iter_all_ensemble_sizes(n_samples, iter, print_all=True):
    import matplotlib.pyplot as plt
    plt.figure()
    plt.plot(list(range(0, n_samples)),
             iter[0][:], 'bo', markersize=8, zorder=1)
    ensemble_size = 8
    for i in range(0, int(np.ceil(n_samples * 1. / ensemble_size))):
        plt.plot(list(range(ensemble_size * i, ensemble_size * (i + 1))),
                 iter[1][ensemble_size * i:ensemble_size * (i + 1)],
                 '-ro',
                 linewidth=3,
                 markersize=6,
                 zorder=2)
    if print_all:
        ensemble_size = 16
        for i in range(0, int(np.ceil(n_samples * 1. / ensemble_size))):
            plt.plot(list(range(ensemble_size * i, ensemble_size * (i + 1))),
                     iter[2][ensemble_size * i:ensemble_size * (i + 1)],
                     '-go',
                     linewidth=2,
                     markersize=4,
                     zorder=2)
        ensemble_size = 32
        for i in range(0, int(np.ceil(n_samples * 1. / ensemble_size))):
            plt.plot(list(range(ensemble_size * i, ensemble_size * (i + 1))),
                     iter[3][ensemble_size * i:ensemble_size * (i + 1)],
                     '-ko',
                     linewidth=1,
                     markersize=2,
                     zorder=3)
    plt.xlabel('sample')
    plt.ylabel('Number of iterations to converge')


def plot_iter_all(base_dir_ER, base_dir_NER, ensemble_sizes, n_samples):

    iter_NER = read_iter_all_ensemble_sizes(n_samples, ensemble_sizes,
                                            base_dir_NER, False)
    iter_ER = read_iter_all_ensemble_sizes(n_samples, ensemble_sizes,
                                           base_dir_ER, True)
    plot_iter_all_ensemble_sizes(n_samples, iter_NER)
    write_tex(n_samples, iter_NER, 'NER_conv.tex')
    np.savetxt('NER_conv.txt', iter_NER)
    np.savetxt('ordering_1.txt',
               np.argsort(iter_NER[0, :]).astype(int),
               fmt='%i')
    np.savetxt('ordering_2.txt',
               np.argsort(iter_NER[0][:]).astype(int),
               fmt='%i')
    plot_iter_all_ensemble_sizes(n_samples, iter_ER)
    write_tex(n_samples, iter_ER, 'ER_conv.tex')
    np.savetxt('ER_conv.txt', iter_ER)

    order = np.argsort(iter_NER[0, :])

    tmp = iter_NER[0][order]

    plt.figure()
    plt.plot(iter_NER[0][:])
    plt.savefig('no_order_1.png', dpi=400)

    iter_NER[0][:] = tmp

    plt.figure()
    plt.plot(tmp)
    plt.savefig('order_1.png', dpi=400)

    tmp = iter_ER[0][order]

    plt.figure()
    plt.plot(iter_ER[0][:])
    plt.savefig('no_order_2.png', dpi=400)

    iter_ER[0][:] = tmp

    write_tex(n_samples, iter_NER, 'NER_conv_2.tex')
    write_tex(n_samples, iter_ER, 'ER_conv_2.tex')

    plt.figure()
    plt.plot(tmp)
    plt.savefig('order_2.png', dpi=400)


def plot_AS_iter_all(base_dir_ER, base_dir_NER, ensemble_sizes, n_samples):

    iter_NER = read_AS_iter_all_ensemble_sizes(n_samples, ensemble_sizes,
                                               base_dir_NER, False)
    iter_ER = read_AS_iter_all_ensemble_sizes(n_samples, ensemble_sizes,
                                              base_dir_ER, True)
    plot_iter_all_ensemble_sizes(n_samples, iter_NER)
    write_tex(n_samples, iter_NER, 'AS_NER_conv.tex')
    np.savetxt('AS_NER_conv.txt', iter_NER)
    np.savetxt('AS_ordering_1.txt',
               np.argsort(iter_NER[0, :]).astype(int),
               fmt='%i')
    np.savetxt('AS_ordering_2.txt',
               np.argsort(iter_NER[0][:]).astype(int),
               fmt='%i')
    plot_iter_all_ensemble_sizes(n_samples, iter_ER)
    write_tex(n_samples, iter_ER, 'AS_ER_conv.tex')
    np.savetxt('AS_ER_conv.txt', iter_ER)

    order = np.argsort(iter_NER[0, :])

    tmp = iter_NER[0][order]

    plt.figure()
    plt.plot(iter_NER[0][:])
    plt.savefig('AS_no_order_1.png', dpi=400)

    iter_NER[0][:] = tmp

    plt.figure()
    plt.plot(tmp)
    plt.savefig('AS_order_1.png', dpi=400)

    tmp = iter_ER[0][order]

    plt.figure()
    plt.plot(iter_ER[0][:])
    plt.savefig('AS_no_order_2.png', dpi=400)

    iter_ER[0][:] = tmp

    write_tex(n_samples, iter_NER, 'AS_NER_conv_2.tex')
    write_tex(n_samples, iter_ER, 'AS_ER_conv_2.tex')

    plt.figure()
    plt.plot(tmp)
    plt.savefig('AS_order_2.png', dpi=400)


def all_post_process(grouped):
    if grouped:
        base_dir_NER = '/Volumes/HD_SONY/Thesis/Test_3/all_results_NER_grouped_2'
        base_dir_ER = '/Volumes/HD_SONY/Thesis/Test_3/all_results_ER_grouped_2'
        directory = 'grouped'
    else:
        base_dir_NER = '/Volumes/HD_SONY/Thesis/Test_3/all_results_NER_beam'
        base_dir_ER = '/Volumes/HD_SONY/Thesis/Test_3/all_results_ER_beam'
        directory = 'not_grouped'
    if not os.path.exists(directory):
        os.makedirs(directory)
    os.chdir(directory)

    N = 640
    ensemble_sizes = [1, 8, 16, 32]

    #compare_QoI(base_dir_NER, base_dir_ER, N, ensemble_sizes)
    speedup_ensemble_reduction(base_dir_ER, base_dir_NER, N)

    E = np.loadtxt(base_dir_NER + '/E_vec.txt')
    p = np.loadtxt(base_dir_NER + '/p_vec.txt')

    qoi_NER_8 = load_qoi(base_dir_NER, N, 8)
    qoi_NER_16 = load_qoi(base_dir_NER, N, 16)
    qoi_NER_32 = load_qoi(base_dir_NER, N, 32)

    qoi_ER_8 = load_qoi(base_dir_ER, N, 8)
    qoi_ER_16 = load_qoi(base_dir_ER, N, 16)
    qoi_ER_32 = load_qoi(base_dir_ER, N, 32)

    np.savetxt('qoi_NER_8.tex', qoi_NER_8)
    '''
    fig = plt.figure()
    ax = fig.gca(projection='3d')    
    ax.scatter(E,p,qoi_ER_8)

    fig = plt.figure()
    ax = fig.gca(projection='3d')    
    ax.scatter(E,p,qoi_ER_16)   

    fig = plt.figure()
    ax = fig.gca(projection='3d')    
    ax.scatter(E,p,qoi_ER_32)        
    '''

    plot_iter_all(base_dir_ER, base_dir_NER, ensemble_sizes, N)
    plot_AS_iter_all(base_dir_ER, base_dir_NER, ensemble_sizes, N)

    plt.figure()
    for i in range(0, 8):
        iter_indices, residuals = read_Belos(
            base_dir_NER + '/1/' + str(i) + '/belos_out.txt', 1)
        plt.semilogy(residuals)

    x = np.array([0, 180])
    y = np.array([1e-6, 1e-6])
    plt.semilogy(x, y, 'k--')

    plt.figure()
    iter_indices, residuals = read_Belos(base_dir_NER + '/8/0/belos_out.txt',
                                         8)
    for i in range(0, 8):
        plt.semilogy(residuals[:, i])

    plt.semilogy(x, y, 'k--')

    plt.figure()
    iter_indices, residuals = read_Belos(base_dir_ER + '/8/0/belos_out.txt', 1)
    plt.semilogy(residuals)
    plt.semilogy(x, y, 'k--')
    os.chdir('..')


def main():
    all_post_process(False)
    all_post_process(True)


if __name__ == "__main__":
    main()
