#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import print_function
from builtins import str
from builtins import range
import numpy as np
import time

from params.src.func import grid 
from params.src.func import next_indices
from params.src.func import phi

from matplotlib import cm
import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D

from fwk.wutils import parseargs


def model(xi,f,d,fig,ax):

    import mirrors.models.mirror_ss as mirror_model

    T_min = 100.
    T_max = 300.

    L0 = 250.0
    L0_1 = 250.0
    H0 = 50
    H0_1 = 30 
    W0 = 150
    nL0_1 = 11 
    nL0 = 11  
    nH0 = 6
    nH0_1 = 6 
    nW0 = 5

    xi[0] = T_min + (T_max - T_min)*(xi[0]+1.)/2.
    sol = mirror_model.runmodel(L = L0, L_1 = L0_1, H = H0, H_1 = H0_1, W = W0, nL_1 = nL0_1, nL = nL0, nH = nH0, nH_1 = nH0_1, nW = nW0, T = xi[0])
    #sol = 1



    f.write('T = ' + str(xi[0]) + '.\n')
    f.write('Sol = ' + str(sol) + '.\n')
    plot(xi,sol,d,fig,ax)

    return sol

def plot(xi,sol,d,fig,ax):
    plt.figure(fig.number)
    if d == 1:
        plt.scatter(xi[0], sol)
    elif d == 2: 
        ax.scatter(xi[0],xi[1], sol)

def main():

    d = 2
    k = 3
    M = np.zeros((d,), dtype =np.int)
    N = np.zeros((d,), dtype =np.int)
    for j in range(0,d):
        M[j] = 3
        N[j] = 3

    
    #----------

    date = time.strftime("%d/%m/%Y")
    hour = time.strftime("%H:%M:%S")

    f = open('params_sol2.log','w')
    f.write('Parametric loop\n')
    f.write('Simulations started at: ' + date + ' ' + hour + '.\n')
    f.write('====================\n')

    X, W = grid(M)  

    print(M)
    print(X)

    m_max = 1
    n_max = 1
    for j in range(0,d):
        m_max = m_max*M[j] 
        n_max = n_max*N[j] 

    print(m_max)
    print(n_max)

    i =      np.zeros((d,), dtype =np.int)
    i_zero = np.zeros((d,), dtype =np.int)
    i_one =  np.ones((d,), dtype =np.int)
    p = 0

    xi = np.zeros((d,))

    fig = plt.figure()
    if d == 1:
        ax = fig.gca()
        #ax.axis([100, 300, 20, 100])
    elif d == 2: 
        ax = fig.gca(projection='3d')
    plt.ion()

    m_i = 0
    sol = np.zeros((m_max,1))

    Z = np.zeros((m_max,n_max))
    W_sr = np.zeros((m_max,m_max))

    while np.all(i != -i_one):
        
        #f.write('----------\n')
        #f.write('i = [')
        #for j in range(0,d-1):
        #    f.write(str(i[j])+ ', ')
        #f.write(str(i[d-1])+ '] =>  ')

        #f.write('xi = [')
        #for j in range(0,d-1):
        #    f.write(str(X[i[j],j])+ ', ')
        #f.write(str(X[i[d-1],d-1])+ ']')

        #f.write('\n')

        name = 'i_'
        for j in range(0,d-1):
            name = name + str(i[j]) + '_'
        name = name + str(i[d-1]) + '_'
        name = name + 'M_' 
        for j in range(0,d-1):
            name = name + str(M[j]) + '_'
        name = name + str(M[d-1]) 

        f.write(name + '\n')

        #print i

        for j in range(0,d):
            xi[j] = X[i[j],j]

        #print xi

        sol[m_i] = model(xi,f,d,fig,ax)
        Z[m_i,:] = np.transpose(phi(xi,N,n_max,d))

        W_sr[m_i,m_i] = 1
        for j in range(0,d):
            W_sr[m_i,m_i] = W_sr[m_i,m_i]*W[i[j],j]

        m_i = m_i + 1

        plt.draw()
        p,i = next_indices(p,i,M,d)
        

    #print sol
    #print Z
    #print W_sr

    W_srZ = np.dot(W_sr,Z)
    U, s, V_t = np.linalg.svd(W_srZ, full_matrices=True)
    S = np.zeros((m_max,n_max))
    for j in range(0,n_max):
        S[j,j] = s[j]

    #print W_srZ - np.dot(U,np.dot(S,V_t))
    s_hat = np.zeros((n_max,))

    V = np.transpose(V_t)

    for j in range(0,n_max):
        s_hat = s_hat + np.dot(np.transpose(U[:,j]),np.dot(W_sr,sol))*V[:,j]/S[j,j] 

    print(s_hat)
    
    #------------------

    m1 = 20
    m2 = 20

    x = np.linspace(100.,300.,m1)
    y = np.linspace(-1.,1.,m2)
    X = np.zeros((m1,m2))
    Y = np.zeros((m1,m2))
    surrogate = np.zeros((m1,m2))
    X, Y = np.meshgrid(x, y)

    for i in range(0,m1):
        for j in range(0,m2):
            xi = np.zeros((2,))
            xi[0] = X[i,j]
            xi[1] = Y[i,j]
            surrogate[i,j] = np.dot(np.transpose(phi(xi,N,n_max,d)),s_hat)

    surf = ax.plot_surface(X, Y, surrogate, rstride=1, cstride=1, cmap=cm.jet,
                       linewidth=0, antialiased=False)

    ax.set_zlim3d(np.amin(sol)-1.,np.amax(sol)+1.)

    date2 = time.strftime("%d/%m/%Y")
    hour2 = time.strftime("%H:%M:%S")
    f.write('====================\n')
    f.write('Simulations finished at: ' + date2 + ' ' + hour2 + '.\n')

    plt.ioff()
    plt.show()
    f.close()

if __name__ == "__main__":
    main()
