/*
 * Copyright 2020 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef WLINE2_H
#define WLINE2_H

#include "tbox.h"
#include "wElement.h"

namespace tbox
{

/**
 * @brief Linear line finite element
 * @authors Romain Boman, Adrien Crovato
 */
class TBOX_API Line2 : public Element
{
#ifndef SWIG
    friend class MemLine2;
    MemLine2 *pmem; ///< private memory of precalculated values
    MemLine2 &getMem();
    static CacheLine2 &getCache();
    virtual double buildJ(size_t k) const override;

    virtual void builds(Eigen::VectorXd &s, std::vector<double> const &u, Fct0 const &f) override;
    virtual void builds(Eigen::VectorXd &s, std::vector<double> const &u, Fct1 const &f) override;

    virtual double computeGrad2(std::vector<double> const &u, size_t k) override;
    virtual double computeV(std::vector<double> const &u, Fct0 const &fct) override;
    virtual double computeV() override;
#endif

public:
    Line2(int n, Tag *_ptag, Tag *_etag, std::vector<Tag *> &_parts, std::vector<Node *> &nods);
    virtual ~Line2();
    virtual ELTYPE type() const override { return ELTYPE::LINE2; }

#ifndef SWIG
    virtual Mem &getVMem() override;
    virtual Cache &getVCache() override;
    virtual Pt normal() override;
#endif
};

} // namespace tbox

#endif //WLINE2_H
