/*
 * Copyright 2020 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "wMemLine2.h"
#include "wNode.h"

using namespace tbox;

MemLine2::MemLine2(Line2 &_e, int _ngp) : Mem(_ngp), e(_e)
{
}

/**
 * @brief Update Line2 values
 * @authors Romain Boman, Adrien Crovato
 */
void MemLine2::update(bool sameDim)
{
    // Line2 is never of same dimension as Problem (surface element)
    for (int k = 0; k < ngp; ++k)
        detJs[k] = e.buildJ(k); // detJ
    // CG
    cg = Pt(0., 0., 0.);
    for (auto n : e.nodes)
        cg += n->pos;
    cg /= e.nodes.size();
    // Memory is now valid and the volume can be computed
    valid = true;
    vol = e.Element::computeV();
}
